@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import contextlib
 import io
 import os
@@ -11,9 +7,11 @@ import pstats
 import sys
 import tempfile
 
-from . import Command
+from asv_runner.console import color_print
+
+from . import Command, common_args
 from ..benchmarks import Benchmarks
-from ..console import log, color_print
+from ..console import log
 from ..environment import get_environments, is_existing_only
 from ..machine import Machine
 from ..profiling import ProfilerGui
@@ -23,8 +21,6 @@ from ..runner import run_benchmarks
 from ..util import hash_equal, iter_subclasses
 from .. import util
 
-from . import common_args
-
 
 @contextlib.contextmanager
 def temp_profile(profile_data):
@@ -52,7 +48,7 @@ class Profile(Command):
             help="""The benchmark to profile.  Must be a
             fully-specified benchmark name. For parameterized benchmark, it
             must include the parameter combination to use, e.g.:
-            benchmark_name(param0, param1, ...)""")
+            benchmark_name\\(param0, param1, ...\\)""")
         parser.add_argument(
             'revision', nargs='?',
             help="""The revision of the project to profile.  May be a
@@ -103,12 +99,12 @@ class Profile(Command):
             log.info("Available profiler GUIs:")
             with log.indent():
                 for x in cls.guis.values():
-                    log.info("{0}: {1}".format(x.name, x.description))
+                    log.info(f"{x.name}: {x.description}")
             return
 
         if gui is not None and gui not in cls.guis:
             raise util.UserError(
-                "Unknown profiler GUI {0}".format(gui))
+                f"Unknown profiler GUI {gui}")
 
         if benchmark is None:
             raise util.UserError(
@@ -132,7 +128,7 @@ class Profile(Command):
         try:
             commit_hash = repo.get_hash_from_name(rev)
         except NoSuchNameError as exc:
-            raise util.UserError("Unknown commit {0}".format(exc))
+            raise util.UserError(f"Unknown commit {exc}")
 
         profile_data = None
         checked_out = set()
@@ -177,17 +173,18 @@ class Profile(Command):
 
             benchmarks = Benchmarks.discover(conf, repo, environments,
                                              [commit_hash],
-                                             regex='^{0}$'.format(benchmark))
+                                             regex=f'^{benchmark}$')
 
             if len(benchmarks) == 0:
-                raise util.UserError("'{0}' benchmark not found".format(benchmark))
+                raise util.UserError(f"'{benchmark}' benchmark not found")
             elif len(benchmarks) > 1:
                 exact_matches = benchmarks.filter_out([x for x in benchmarks if x != benchmark])
                 if len(exact_matches) == 1:
-                    log.warning("'{0}' matches more than one benchmark, using exact match".format(benchmark))
+                    log.warning("'{0}' matches more than one benchmark, "
+                                "using exact match".format(benchmark))
                     benchmarks = exact_matches
                 else:
-                    raise util.UserError("'{0}' matches more than one benchmark".format(benchmark))
+                    raise util.UserError(f"'{benchmark}' matches more than one benchmark")
 
             benchmark_name, = benchmarks.keys()
 
@@ -209,7 +206,7 @@ class Profile(Command):
         log.flush()
 
         if gui is not None:
-            log.debug("Opening gui {0}".format(gui))
+            log.debug(f"Opening gui {gui}")
             with temp_profile(profile_data) as profile_path:
                 return cls.guis[gui].open_profiler_gui(profile_path)
         elif output is not None:
@@ -219,5 +216,6 @@ class Profile(Command):
             color_print('')
             with temp_profile(profile_data) as profile_path:
                 stats = pstats.Stats(profile_path)
+                stats.strip_dirs()  # Addresses gh-71
                 stats.sort_stats('cumulative')
                 stats.print_stats()
