@@ -1,9 +1,4 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import os
 import sys
 import logging
@@ -11,27 +6,19 @@ import traceback
 import time
 import argparse
 import textwrap
-
 from collections import defaultdict
 
-import six
-
-from . import Command
+from . import Command, common_args
 from ..benchmarks import Benchmarks
 from ..console import log
 from ..machine import Machine
 from ..repo import get_repo, NoSuchNameError
-from ..results import (Results, get_existing_hashes,
-                       iter_results_for_machine_and_hash)
+from ..results import Results, get_existing_hashes, iter_results_for_machine_and_hash
 from ..runner import run_benchmarks, skip_benchmarks
-from .. import environment
-from .. import util
-
+from .. import environment, util
 from .setup import Setup
 from .show import Show
 
-from . import common_args
-
 
 def _do_build(args):
     env, conf, repo, commit_hash = args
@@ -173,10 +160,6 @@ class Run(Command):
         parser.add_argument(
             "--no-pull", action="store_true",
             help="Do not pull the repository")
-        parser.add_argument(
-            "--strict", action="store_true",
-            help="When set true the run command will exit with a non-zero "
-                 "return code if any benchmark is in a failed state")
 
     @classmethod
     def run_from_conf_args(cls, conf, args, **kwargs):
@@ -192,7 +175,7 @@ class Run(Command):
             record_samples=args.record_samples, append_samples=args.append_samples,
             pull=not args.no_pull, interleave_rounds=args.interleave_rounds,
             launch_method=args.launch_method, durations=args.durations,
-            strict=args.strict, **kwargs
+            **kwargs
         )
 
     @classmethod
@@ -202,7 +185,7 @@ class Run(Command):
             dry_run=False, machine=None, _machine_file=None, skip_successful=False,
             skip_failed=False, skip_existing_commits=False, record_samples=False,
             append_samples=False, pull=True, interleave_rounds=False,
-            launch_method=None, durations=0, strict=False, _returns={}):
+            launch_method=None, durations=0, _returns={}):
         machine_params = Machine.load(
             machine_name=machine,
             _path=_machine_file, interactive=True)
@@ -221,8 +204,8 @@ class Run(Command):
             if dry_run:
                 raise util.UserError("--interleave-rounds and --dry-run cannot be used together")
             if has_existing_env:
-                raise util.UserError("--interleave-rounds cannot be used with existing environment "
-                                     "(or python=same)")
+                raise util.UserError("--interleave-rounds cannot be used with existing "
+                                     "environment (or python=same)")
         elif interleave_rounds is None:
             # Enable if possible
             interleave_rounds = not (dry_run or has_existing_env)
@@ -245,9 +228,10 @@ class Run(Command):
 
         if range_spec is None:
             try:
-                commit_hashes = list(set([repo.get_hash_from_name(branch) for branch in conf.branches]))
+                commit_hashes = list(set([repo.get_hash_from_name(branch) for
+                                         branch in conf.branches]))
             except NoSuchNameError as exc:
-                raise util.UserError('Unknown branch {0} in configuration'.format(exc))
+                raise util.UserError(f'Unknown branch {exc} in configuration')
         elif range_spec == 'EXISTING':
             commit_hashes = get_existing_hashes(conf.results_dir)
         elif range_spec == "NEW":
@@ -257,7 +241,7 @@ class Run(Command):
         elif range_spec == "ALL":
             # All commits on each configured branches
             commit_hashes = repo.get_new_branch_commits(conf.branches, [])
-        elif isinstance(range_spec, six.string_types) and range_spec.startswith('HASHFILE:'):
+        elif isinstance(range_spec, str) and range_spec.startswith('HASHFILE:'):
             hashfn = range_spec[9:]
             if hashfn == '-':
                 hashstr = sys.stdin.read()
@@ -265,7 +249,7 @@ class Run(Command):
                 with open(hashfn, 'r') as f:
                     hashstr = f.read()
             else:
-                log.error('Requested commit hash file "{}" is not a file'.format(hashfn))
+                log.error(f'Requested commit hash file "{hashfn}" is not a file')
                 return 1
             commit_hashes = []
             for h in hashstr.split("\n"):
@@ -274,7 +258,7 @@ class Run(Command):
                     try:
                         commit_hashes.append(repo.get_hash_from_name(h))
                     except NoSuchNameError:
-                        log.warning("Unknown commit hash {0} in input file".format(h))
+                        log.warning(f"Unknown commit hash {h} in input file")
         elif isinstance(range_spec, list):
             commit_hashes = range_spec
         else:
@@ -317,10 +301,9 @@ class Run(Command):
         steps = len(commit_hashes) * benchmark_count * len(environments)
 
         log.info(
-            "Running {0} total benchmarks "
-            "({1} commits * {2} environments * {3} benchmarks)".format(
-                steps, len(commit_hashes),
-                len(environments), len(benchmarks)))
+            f"Running {steps} total benchmarks "
+            f"({len(commit_hashes)} commits * {len(environments)} "
+            f"environments * {len(benchmarks)} benchmarks)")
 
         parallel, multiprocessing = util.get_multiprocessing(parallel)
 
@@ -332,7 +315,7 @@ class Run(Command):
             max_rounds = int(attribute['rounds'])
         else:
             max_rounds = max(b.get('rounds', 1)
-                             for b in six.itervalues(benchmarks))
+                             for b in benchmarks.values())
 
         log.set_nitems(steps * max_rounds)
 
@@ -360,7 +343,7 @@ class Run(Command):
                                 skipped_benchmarks[(commit_hash, result.env_name)].add(key)
                             if skip_successful and not failed:
                                 skipped_benchmarks[(commit_hash, result.env_name)].add(key)
-                except IOError:
+                except OSError:
                     pass
 
         if interleave_rounds:
@@ -401,7 +384,7 @@ class Run(Command):
                                 log.step()
 
             active_environments = [env for env in environments
-                                   if set(six.iterkeys(benchmarks))
+                                   if set(benchmarks.keys())
                                    .difference(skipped_benchmarks[(commit_hash, env.name)])]
 
             if not active_environments:
@@ -409,16 +392,13 @@ class Run(Command):
 
             if commit_hash:
                 if interleave_rounds:
-                    round_info = " (round {}/{})".format(
-                        max_rounds - run_rounds[0] + 1,
-                        max_rounds)
+                    round_info = f" (round {max_rounds - run_rounds[0] + 1}/{max_rounds})"
                 else:
                     round_info = ""
 
                 commit_name = repo.get_decorated_hash(commit_hash, 8)
                 log.info(
-                    "For {0} commit {1}{2}:".format(
-                        conf.project, commit_name, round_info))
+                    f"For {conf.project} commit {commit_name}{round_info}:")
 
             with log.indent():
 
@@ -433,7 +413,7 @@ class Run(Command):
                     subenv_name = ', '.join([x.name for x in env_to_install])
 
                     if subenv_name:
-                        log.info("Building for {0}".format(subenv_name))
+                        log.info(f"Building for {subenv_name}")
 
                     with log.indent():
                         args = [(env, conf, repo, commit_hash) for env in env_to_install]
@@ -472,8 +452,9 @@ class Run(Command):
                         params['python'] = env.python
                         params.update(env.requirements)
 
-                        skip_save = dry_run or (isinstance(env, environment.ExistingEnvironment)
-                                                and set_commit_hash is None)
+                        skip_save = (dry_run or
+                                     (isinstance(env, environment.ExistingEnvironment) and
+                                      set_commit_hash is None))
 
                         skip_list = skipped_benchmarks[(commit_hash, env.name)]
                         benchmark_set = benchmarks.filter_out(skip_list)
@@ -521,15 +502,15 @@ class Run(Command):
                         if not skip_save:
                             result.save(conf.results_dir)
 
-                        if strict:
-                            failures = failures or any(
-                                code != 0 for code in result.errcode.values())
+                        failures = failures or any(
+                            code != 0 for code in result.errcode.values())
 
                         if durations > 0:
                             duration_set = Show._get_durations([(machine, result)], benchmark_set)
-                            log.info(cls.format_durations(duration_set[(machine, env.name)], durations))
+                            log.info(cls.format_durations(
+                                duration_set[(machine, env.name)], durations))
 
-        if failures and strict:
+        if failures:
             return 2
 
     @classmethod
