@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 """
@@ -6,24 +5,18 @@ Manages an environment -- a combination of a version of Python and set
 of dependencies.
 """
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 import hashlib
 import os
 import re
-import shutil
-import shlex
 import sys
 import itertools
 import subprocess
-
-import six
+import importlib
+from pathlib import Path
 
 from .console import log
-from . import util
-from . import build_cache
-
+from . import util, build_cache
 
 WIN = (os.name == "nt")
 
@@ -213,7 +206,7 @@ def _parse_exclude_include_rule(rule, is_include=False):
         Dictionary of {(key_type, key): value, ...}
     """
     if is_include and 'python' not in rule:
-        raise util.UserError("include rule '{0}' does not specify Python version".format(rule))
+        raise util.UserError(f"include rule '{rule}' does not specify Python version")
 
     bare_keys = ('python', 'environment_type', 'sys_platform')
     return _parse_matrix(rule, bare_keys)
@@ -274,8 +267,8 @@ def get_env_name(tool_name, python, requirements, tagged_env_vars, build=False):
         # Backward compatibility vs. result file names
         name = []
 
-    name.append("py{0}".format(python))
-    reqs = list(six.iteritems(requirements))
+    name.append(f"py{python}")
+    reqs = list(requirements.items())
     reqs.sort()
     for key, val in reqs:
         if val:
@@ -285,7 +278,7 @@ def get_env_name(tool_name, python, requirements, tagged_env_vars, build=False):
 
     env_vars = _untag_env_vars(tagged_env_vars, build=build)
 
-    for env_var, value in sorted(six.iteritems(env_vars)):
+    for env_var, value in sorted(env_vars.items()):
         name.append(''.join([env_var, value]))
 
     return util.sanitize_filename('-'.join(name))
@@ -294,7 +287,7 @@ def get_env_name(tool_name, python, requirements, tagged_env_vars, build=False):
 def _untag_env_vars(tagged_env_vars, build=False):
     vars = {}
 
-    for (tag, key), value in six.iteritems(tagged_env_vars):
+    for (tag, key), value in tagged_env_vars.items():
         if not build or tag == 'build':
             vars[key] = value
 
@@ -397,7 +390,7 @@ def _parse_matrix_entries(entries):
             requirements[key] = value
         else:
             # Shouldn't happen
-            raise ValueError("Invalid matrix key type {0}".format(key))
+            raise ValueError(f"Invalid matrix key type {key}")
     return python, requirements, tagged_env_vars
 
 
@@ -437,7 +430,7 @@ def get_environment_class(conf, python):
         if cls.matches_python_fallback and cls.matches(python):
             return cls
     raise EnvironmentUnavailable(
-        "No way to create environment for python='{0}'".format(python))
+        f"No way to create environment for python='{python}'")
 
 
 def get_environment_class_by_name(environment_type):
@@ -448,7 +441,7 @@ def get_environment_class_by_name(environment_type):
         if cls.tool_name == environment_type:
             return cls
     raise EnvironmentUnavailable(
-        "Unknown environment type '{0}'".format(environment_type))
+        f"Unknown environment type '{environment_type}'")
 
 
 def is_existing_only(environments):
@@ -462,7 +455,7 @@ class EnvironmentUnavailable(BaseException):
     pass
 
 
-class Environment(object):
+class Environment:
     """
     Manage a single environment -- a combination of a particular
     version of Python and a set of dependencies for the benchmarked
@@ -500,7 +493,8 @@ class Environment(object):
         """
         self._env_dir = conf.env_dir
         self._repo_subdir = conf.repo_subdir
-        self._install_timeout = conf.install_timeout  # GH391
+        self._install_timeout = conf.install_timeout  # gh-391
+        self._default_benchmark_timeout = conf.default_benchmark_timeout # gh-973
         self._tagged_env_vars = tagged_env_vars
         self._path = os.path.abspath(os.path.join(
             self._env_dir, self.dir_name))
@@ -511,6 +505,47 @@ class Environment(object):
         self._cache = build_cache.BuildCache(conf, self._path)
         self._build_root = os.path.abspath(os.path.join(self._path, 'project'))
 
+        self._requirements = requirements
+        # These are needed for asv to build and run the project, not part of
+        # benchmark name mangling
+        self._base_requirements = {}
+        # gh-1314
+        asv_runner_path = os.getenv("ASV_RUNNER_PATH", "")
+        module_path = Path(asv_runner_path) / "asv_runner"
+
+        # Check if the path points to a directory containing the "asv_runner" module
+        if module_path.is_dir() and (module_path / "__init__.py").is_file():
+            spec = importlib.util.spec_from_file_location("asv_runner",
+                                                          module_path / "__init__.py")
+            # Attempt to load the module
+            asv_runner_module = importlib.util.module_from_spec(spec)
+            try:
+                spec.loader.exec_module(asv_runner_module)
+                self._base_requirements["pip+asv_runner"] = asv_runner_path
+            except Exception as e:
+                self._base_requirements["pip+asv_runner"] = ""
+                log.warning(f"Failed to load module from ASV_RUNNER_PATH: {e}")
+        else:
+            self._base_requirements["pip+asv_runner"] = ""
+            if asv_runner_path:
+                log.warning("ASV_RUNNER_PATH does not point"
+                            "to a directory containing the 'asv_runner' module")
+        if not util.ON_PYPY:
+            # XXX: What if pypy installed asv tries to benchmark a cpython
+            # python?
+            self._base_requirements["pip+pympler"] = ""
+        if (Path.cwd() / "poetry.lock").exists():
+            self._base_requirements["poetry-core"] = ""
+
+        if (Path.cwd() / "pdm.lock").exists():
+            self._base_requirements["pdm"] = ""
+
+        # Update the _base_requirements if needed
+        for key in list(self._requirements.keys()):
+            if key in self._base_requirements:
+                self._base_requirements[key] = self._requirements[key]
+                del self._requirements[key]
+
         self._build_command = conf.build_command
         self._install_command = conf.install_command
         self._uninstall_command = conf.uninstall_command
@@ -557,7 +592,7 @@ class Environment(object):
         if os.path.isfile(hash_file):
             try:
                 data = util.load_json(hash_file, api_version=1)
-            except util.UserError as exc:
+            except util.UserError:
                 pass
 
         # If configuration changed, force reinstall
@@ -620,6 +655,7 @@ class Environment(object):
 
     @property
     def requirements(self):
+        """Return the requirements"""
         return self._requirements
 
     @property
@@ -649,7 +685,7 @@ class Environment(object):
 
         try:
             info = self.load_info_file(self._path)
-        except (util.UserError, IOError):
+        except (util.UserError, OSError):
             return False
 
         expected_info = {
@@ -665,7 +701,7 @@ class Environment(object):
         for executable in ['pip', 'python']:
             try:
                 self.find_executable(executable)
-            except IOError:
+            except OSError:
                 return False
 
         try:
@@ -702,8 +738,8 @@ class Environment(object):
 
             try:
                 self._setup()
-            except:
-                log.error("Failure creating environment for {0}".format(self.name))
+            except Exception:
+                log.error(f"Failure creating environment for {self.name}")
                 if os.path.exists(self._path):
                     util.long_path_rmtree(self._path)
                 raise
@@ -845,7 +881,7 @@ class Environment(object):
 
         if cmd:
             commit_name = repo.get_decorated_hash(commit_hash, 8)
-            log.info("Installing {0} into {1}".format(commit_name, self.name))
+            log.info(f"Installing {commit_name} into {self.name}")
             self._interpolate_and_run_commands(cmd, default_cwd=build_dir,
                                                extra_env=self.build_env_vars)
 
@@ -863,7 +899,7 @@ class Environment(object):
             cmd = ['return-code=any python -mpip uninstall -y {project}']
 
         if cmd:
-            log.info("Uninstalling from {0}".format(self.name))
+            log.info(f"Uninstalling from {self.name}")
             self._interpolate_and_run_commands(cmd, default_cwd=self._env_dir,
                                                extra_env=self.build_env_vars)
 
@@ -871,15 +907,30 @@ class Environment(object):
         """
         Run build commands
         """
+        build_dir_path = Path(build_dir)
+
+        def has_file(file_name):
+            return (build_dir_path / file_name).exists()
+
         cmd = self._build_command
+
         if cmd is None:
-            cmd = ["python setup.py build",
-                   ("PIP_NO_BUILD_ISOLATION=false "
-                    "python -mpip wheel --no-deps --no-index -w {build_cache_dir} {build_dir}")]
+            if has_file('pyproject.toml'):
+                cmd = [
+                    ("PIP_NO_BUILD_ISOLATION=false "
+                     "python -mpip wheel --no-deps --no-index "
+                     "-w {build_cache_dir} {build_dir}")
+                ]
+            else:
+                cmd = [
+                    "python setup.py build",
+                    ("PIP_NO_BUILD_ISOLATION=false "
+                     "python -mpip wheel --no-deps --no-index -w {build_cache_dir} {build_dir}")
+                ]
 
         if cmd:
             commit_name = repo.get_decorated_hash(commit_hash, 8)
-            log.info("Building {0} for {1}".format(commit_name, self.name))
+            log.info(f"Building {commit_name} for {self.name}")
             self._interpolate_and_run_commands(cmd, default_cwd=build_dir,
                                                extra_env=self.build_env_vars)
 
@@ -894,7 +945,7 @@ class Environment(object):
         """
         Find an executable (eg. python, pip) in the environment.
 
-        If not found, raises IOError
+        If not found, raises OSError
         """
 
         # Assume standard virtualenv/Conda layout
@@ -946,6 +997,8 @@ class Environment(object):
                              PIP_USER=str("false"),
                              PATH=str(os.pathsep.join(paths)))
         exe = self.find_executable(executable)
+        if kwargs.get("timeout", None) is None:
+            kwargs["timeout"] = self._install_timeout
         return util.check_output([exe] + args, **kwargs)
 
     def load_info_file(self, path):
@@ -983,7 +1036,7 @@ class ExistingEnvironment(Environment):
                  'import sys; '
                  'print(str(sys.version_info[0]) + "." + str(sys.version_info[1]))'
                  ]).strip()
-        except (util.ProcessError, OSError, IOError):
+        except (util.ProcessError, OSError):
             raise EnvironmentUnavailable()
 
         self._executable = executable
@@ -1006,7 +1059,7 @@ class ExistingEnvironment(Environment):
 
         try:
             util.which(python)
-        except IOError:
+        except OSError:
             return False
         else:
             return True
@@ -1034,6 +1087,6 @@ class ExistingEnvironment(Environment):
         return False
 
     def run(self, args, **kwargs):
-        log.debug("Running '{0}' in {1}".format(' '.join(args), self.name))
+        log.debug(f"Running '{' '.join(args)}' in {self.name}")
         return util.check_output([
             self._executable] + args, **kwargs)
