@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import datetime
 
 from . import util
@@ -16,7 +12,7 @@ class NoSuchNameError(RuntimeError):
     pass
 
 
-class Repo(object):
+class Repo:
     """
     Base class for repository handlers.
 
@@ -48,12 +44,12 @@ class Repo(object):
         Internal routine for raising an error message if mirror directory already exists,
         but appears incorrect.
         """
-        raise util.UserError("Directory '{0}' already exists, but is not a mirror "
+        raise util.UserError(f"Directory '{path}' already exists, but is not a mirror "
                              "of the project repository. Please remove it and try again. "
                              "If the benchmark suite is in the project repository, you can "
                              "also adjust the configuration to use the current "
                              "repository (e.g. \"repo\": \".\") instead of a remote URL "
-                             "as the source.".format(path))
+                             "as the source.")
 
     def checkout(self, path, commit_hash):
         """
@@ -97,7 +93,7 @@ class Repo(object):
 
     def get_new_range_spec(self, latest_result, branch=None):
         """
-        Returns a formatted string giving the results between the 
+        Returns a formatted string giving the results between the
         latest result and the newest hash in a given branch.
         If no branch given, use the 'master' branch.
         """
@@ -144,7 +140,7 @@ class Repo(object):
         """
         name = self.get_name_from_hash(commit)
         if name is not None:
-            return "{0} <{1}>".format(commit[:hash_length], name)
+            return f"{commit[:hash_length]} <{name}>"
         else:
             return commit[:hash_length]
 
@@ -257,7 +253,7 @@ class NoRepository(Repo):
 
     def get_date(self, hash):
         if hash is None:
-            return datetime.datetime.now()
+            return datetime.datetime.now(datetime.timezone.utc)
         self._raise_error()
 
     def get_hashes_from_range(self, range):
@@ -288,6 +284,7 @@ class NoRepository(Repo):
     def get_new_branch_commits(self, branches, existing):
         self._raise_error()
 
+
 def get_repo(conf):
     """
     Get a Repo subclass for the given configuration.
@@ -309,4 +306,4 @@ def get_repo(conf):
                 return cls(conf.repo, conf.project)
 
     raise util.UserError(
-        "Can not determine what kind of DVCS to use for URL '{0}'".format(conf.repo))
+        f"Can not determine what kind of DVCS to use for URL '{conf.repo}'")
