@@ -1,10 +1,5 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import sys
 import base64
 import os
 import re
@@ -12,15 +7,14 @@ import zlib
 import itertools
 import hashlib
 import datetime
+import tempfile
+import pstats
 
-import six
-from six.moves import zip as izip
+from asv_runner.statistics import get_err, compute_stats
 
-from . import environment
+from . import environment, util
 from .console import log
 from .machine import Machine
-from . import statistics
-from . import util
 
 
 def iter_results_paths(results):
@@ -36,13 +30,12 @@ def iter_results_paths(results):
         try:
             data = util.load_json(machine_json, api_version=Machine.api_version)
             machine_name = data.get('machine')
-            if not isinstance(machine_name, six.text_type):
-                raise util.UserError("malformed {0}".format(machine_json))
+            if not isinstance(machine_name, str):
+                raise util.UserError(f"malformed {machine_json}")
         except util.UserError as err:
-            machine_json_err = "Skipping results: {0}".format(six.text_type(err))
-        except IOError as err:
-            machine_json_err = "Skipping results: could not load {0}".format(
-                machine_json)
+            machine_json_err = f"Skipping results: {err}"
+        except OSError:
+            machine_json_err = f"Skipping results: could not load {machine_json}"
         else:
             machine_json_err = None
 
@@ -65,7 +58,7 @@ def iter_results(results):
         try:
             yield Results.load(os.path.join(root, filename), machine_name=machine_name)
         except util.UserError as exc:
-            log.warning(six.text_type(exc))
+            log.warning(str(exc))
 
 
 def iter_results_for_machine(results, machine_name):
@@ -89,7 +82,7 @@ def iter_results_for_machine_and_hash(results, machine_name, commit):
             try:
                 yield Results.load(os.path.join(root, filename), machine_name=machine_name)
             except util.UserError as exc:
-                log.warning(six.text_type(exc))
+                log.warning(str(exc))
 
 
 def iter_existing_hashes(results):
@@ -127,8 +120,8 @@ def get_result_hash_from_prefix(results, machine_name, commit_prefix):
 
     for (root, filename, r_machine_name) in iter_results_paths(path):
         if r_machine_name != machine_name:
-            log.warning("Skipping results '{0}': machine name is not '{1}'".format(
-                os.path.join(root, filename), machine_name))
+            log.warning(f"Skipping results '{os.path.join(root, filename)}':"
+                        f" machine name is not '{machine_name}'")
             continue
 
         results_commit = filename.split('-')[0]
@@ -138,8 +131,8 @@ def get_result_hash_from_prefix(results, machine_name, commit_prefix):
 
     if len(commits) > 1:
         commit_list_str = ', '.join(sorted(commits))
-        raise util.UserError('Git hash prefix could represent one of ' +
-                             'multiple commits: {0}'.format(commit_list_str))
+        raise util.UserError('Git hash prefix could represent one of '
+                             f'multiple commits: {commit_list_str}')
     elif len(commits) == 1:
         return list(commits)[0]
     else:
@@ -158,9 +151,7 @@ def get_filename(machine, commit_hash, env_name):
 
     return os.path.join(
         machine,
-        "{0}-{1}.json".format(
-            commit_hash[:8],
-            env_name))
+        f"{commit_hash[:8]}-{env_name}.json")
 
 
 def _compatible_results(result, result_params, params):
@@ -175,7 +166,7 @@ def _compatible_results(result, result_params, params):
     # Pick results for those parameters that also appear in the
     # current benchmark
     old_results = {}
-    for param, value in izip(itertools.product(*result_params), result):
+    for param, value in zip(itertools.product(*result_params), result):
         old_results[param] = value
 
     new_results = []
@@ -185,14 +176,21 @@ def _compatible_results(result, result_params, params):
     return new_results
 
 
-class Results(object):
+class Results:
     """
     Manage a set of benchmark results for a single machine and commit
     hash.
     """
-    api_version = 1
+    api_version = 2
 
-    def __init__(self, params, requirements, commit_hash, date, python, env_name):
+    def __init__(self,
+                 params,
+                 requirements,
+                 commit_hash,
+                 date,
+                 python,
+                 env_name,
+                 env_vars):
         """
         Parameters
         ----------
@@ -215,6 +213,9 @@ class Results(object):
 
         env_name : str
             Environment name
+
+        env_vars: dict
+            Environment variables
         """
         self._params = params
         self._requirements = requirements
@@ -228,8 +229,9 @@ class Results(object):
         self._python = python
         self._env_name = env_name
         self._started_at = {}
-        self._ended_at = {}
+        self._duration = {}
         self._benchmark_version = {}
+        self._env_vars = env_vars
 
         # Note: stderr and errcode are not saved to files
         self._stderr = {}
@@ -243,7 +245,7 @@ class Results(object):
 
     @classmethod
     def unnamed(cls):
-        return cls({}, {}, None, None, None, None)
+        return cls({}, {}, None, None, None, None, {})
 
     @property
     def commit_hash(self):
@@ -257,13 +259,23 @@ class Results(object):
     def params(self):
         return self._params
 
+    @property
+    def env_vars(self):
+        return self._env_vars
+
     @property
     def started_at(self):
         return self._started_at
 
     @property
-    def ended_at(self):
-        return self._ended_at
+    def duration(self):
+        return self._duration
+
+    def set_build_duration(self, value):
+        self._duration["<build>"] = float(value)
+
+    def set_setup_cache_duration(self, setup_cache_key, value):
+        self._duration[f"<setup_cache {setup_cache_key}>"] = float(value)
 
     @property
     def benchmark_version(self):
@@ -281,7 +293,7 @@ class Results(object):
         """
         Return all available result keys.
         """
-        return six.iterkeys(self._results)
+        return self._results.keys()
 
     def get_result_keys(self, benchmarks):
         """
@@ -300,7 +312,7 @@ class Results(object):
 
         """
         keys = set()
-        for key in six.iterkeys(self._results):
+        for key in self._results.keys():
             if key not in benchmarks:
                 continue
 
@@ -398,7 +410,7 @@ class Results(object):
 
         # Remove run times (may be missing in old files)
         self._started_at.pop(key, None)
-        self._ended_at.pop(key, None)
+        self._duration.pop(key, None)
 
         # Remove version (may be missing)
         self._benchmark_version.pop(key, None)
@@ -417,7 +429,7 @@ class Results(object):
                 self._samples[key][j] = None
 
     def add_result(self, benchmark, result,
-                   started_at=None, ended_at=None,
+                   started_at=None, duration=None,
                    record_samples=False,
                    append_samples=False,
                    selected_idx=None):
@@ -435,8 +447,8 @@ class Results(object):
         started_at : datetime.datetime, optional
             Benchmark start time.
 
-        ended_at : datetime.datetime, optional
-            Benchmark end time.
+        duration : float, optional
+            Benchmark total duration in seconds.
 
         record_samples : bool, optional
             Whether to save samples.
@@ -456,10 +468,7 @@ class Results(object):
         benchmark_version = benchmark['version']
 
         if started_at is None:
-            started_at = datetime.datetime.utcnow()
-
-        if ended_at is None:
-            ended_at = started_at
+            started_at = datetime.datetime.now(datetime.timezone.utc)
 
         new_stats = [None] * len(new_result)
 
@@ -493,7 +502,7 @@ class Results(object):
                 continue
 
             if n is not None:
-                new_result[j], new_stats[j] = statistics.compute_stats(s, n)
+                new_result[j], new_stats[j] = compute_stats(s, n)
 
         # Compress None lists to just None
         if all(x is None for x in new_result):
@@ -514,7 +523,10 @@ class Results(object):
 
         self._benchmark_params[benchmark_name] = benchmark['params'] if benchmark['params'] else []
         self._started_at[benchmark_name] = util.datetime_to_js_timestamp(started_at)
-        self._ended_at[benchmark_name] = util.datetime_to_js_timestamp(ended_at)
+        if duration is None:
+            self._duration.pop(benchmark_name, None)
+        else:
+            self._duration[benchmark_name] = float(duration)
         self._benchmark_version[benchmark_name] = benchmark_version
 
         self._stderr[benchmark_name] = result.stderr
@@ -522,10 +534,17 @@ class Results(object):
 
         if result.profile:
             profile_data = base64.b64encode(zlib.compress(result.profile))
-            if sys.version_info[0] >= 3:
-                profile_data = profile_data.decode('ascii')
+            profile_data = profile_data.decode('ascii')
             self._profiles[benchmark_name] = profile_data
 
+    def _mk_pstats(self, bytedata):
+        fd, fpath = tempfile.mkstemp()
+        with os.fdopen(fd, 'wb') as hp:
+            hp.write(bytedata)
+        pstat = pstats.Stats(fpath)
+        os.remove(fpath)
+        return pstat
+
     def get_profile(self, benchmark_name):
         """
         Get the profile data for the given benchmark name.
@@ -537,14 +556,18 @@ class Results(object):
 
         Returns
         -------
-        profile_data : bytes
-            Raw profile data
+        profile_data : pstats.Stats
+            Profile data
 
         """
         profile_data = self._profiles[benchmark_name]
-        if sys.version_info[0] >= 3:
-            profile_data = profile_data.encode('ascii')
-        return zlib.decompress(base64.b64decode(profile_data))
+        profile_data = profile_data.encode('ascii')
+        profile_bytes = zlib.decompress(base64.b64decode(profile_data))
+        return profile_bytes
+
+    def get_profile_stats(self, benchmark_name):
+        profile_bytes = self.get_profile(benchmark_name)
+        return self._mk_pstats(profile_bytes)
 
     def has_profile(self, benchmark_name):
         """
@@ -567,33 +590,65 @@ class Results(object):
         path = os.path.join(result_dir, self._filename)
 
         results = {}
-        for key in six.iterkeys(self._samples):
-            # Save omitting default values
-            value = {'result': self._results[key]}
-            if self._samples[key] and any(x is not None for x in self._samples[key]):
-                value['samples'] = self._samples[key]
-            if self._stats[key] and any(x is not None for x in self._stats[key]):
-                value['stats'] = self._stats[key]
-            if self._benchmark_params[key]:
-                value['params'] = self._benchmark_params[key]
-            if list(value.keys()) == ['result']:
-                value = value['result']
-                if isinstance(value, list) and len(value) == 1:
-                    value = value[0]
-            results[key] = value
+
+        simple_dict = {
+            'result': self._results,
+            'params': self._benchmark_params,
+            'version': self._benchmark_version,
+            'started_at': self._started_at,
+            'duration': self._duration,
+            'samples': self._samples,
+            'profile': self._profiles,
+        }
+        all_keys = ['result', 'params', 'version', 'started_at', 'duration',
+                    'stats_ci_99_a', 'stats_ci_99_b', 'stats_q_25', 'stats_q_75',
+                    'stats_number', 'stats_repeat', 'samples', 'profile']
+
+        for name in self._results.keys():
+            row = []
+
+            for key in all_keys:
+                if key in simple_dict:
+                    value = simple_dict[key].get(name)
+                else:
+                    assert key[:6] == 'stats_'
+                    z = self._stats[name]
+                    if z is None:
+                        value = None
+                    else:
+                        value = [x.get(key[6:]) if x is not None else None
+                                 for x in z]
+
+                if key != 'params':
+                    if isinstance(value, list) and all(x is None for x in value):
+                        value = None
+
+                if key.startswith('stats_') or key == 'duration':
+                    value = util.truncate_float_list(value)
+
+                row.append(value)
+
+            while row and row[-1] is None:
+                row.pop()
+
+            results[name] = row
+
+        other_durations = {}
+        for key, value in self._duration.items():
+            if key.startswith('<'):
+                other_durations[key] = value
 
         data = {
-            'results': results,
-            'params': self._params,
-            'requirements': self._requirements,
             'commit_hash': self._commit_hash,
-            'date': self._date,
             'env_name': self._env_name,
+            'date': self._date,
+            'params': self._params,
             'python': self._python,
-            'profiles': self._profiles,
-            'started_at': self._started_at,
-            'ended_at': self._ended_at,
-            'benchmark_version': self._benchmark_version,
+            'requirements': self._requirements,
+            'env_vars': self._env_vars,
+            'result_columns': all_keys,
+            'results': results,
+            'durations': other_durations,
         }
 
         util.write_json(path, data, self.api_version, compact=True)
@@ -609,9 +664,9 @@ class Results(object):
 
         if os.path.isfile(path):
             old = self.load(path)
-            for dict_name in ('_results', '_samples', '_stats',
+            for dict_name in ('_results', '_samples', '_stats', '_env_vars',
                               '_benchmark_params', '_profiles', '_started_at',
-                              '_ended_at', '_benchmark_version'):
+                              '_duration', '_benchmark_version'):
                 setattr(self, dict_name, getattr(old, dict_name))
 
     @classmethod
@@ -628,6 +683,7 @@ class Results(object):
 
         """
         d = util.load_json(path, cls.api_version)
+        d.setdefault('env_vars', {})
 
         try:
             obj = cls(
@@ -636,16 +692,109 @@ class Results(object):
                 d['commit_hash'],
                 d['date'],
                 d['python'],
-                d.get('env_name',
-                      environment.get_env_name('', d['python'], d['requirements']))
+                d['env_name'],
+                d['env_vars'],
             )
 
             obj._results = {}
             obj._samples = {}
             obj._stats = {}
             obj._benchmark_params = {}
+            obj._profiles = {}
+            obj._started_at = {}
+            obj._duration = d.get('durations', {})
+            obj._benchmark_version = {}
 
-            for key, value in six.iteritems(d['results']):
+            simple_keys = {
+                'result': obj._results,
+                'params': obj._benchmark_params,
+                'version': obj._benchmark_version,
+                'started_at': obj._started_at,
+                'duration': obj._duration,
+                'samples': obj._samples,
+                'profile': obj._profiles,
+            }
+
+            for name, key_values in d['results'].items():
+                for key, value in zip(d['result_columns'], key_values):
+                    key_dict = simple_keys.get(key)
+                    if key_dict is not None:
+                        key_dict[name] = value
+                        continue
+                    elif key.startswith('stats_'):
+                        if value is not None:
+                            if name not in obj._stats:
+                                obj._stats[name] = [{} for _ in value]
+
+                            stats_key = key[6:]
+                            for j, v in enumerate(value):
+                                if v is not None:
+                                    obj._stats[name][j][stats_key] = v
+                    else:
+                        raise KeyError(f"unknown data key {key}")
+
+                for key_dict in simple_keys.values():
+                    key_dict.setdefault(name, None)
+                obj._stats.setdefault(name, None)
+
+            obj._filename = os.path.join(*path.split(os.path.sep)[-2:])
+        except KeyError as exc:
+            raise util.UserError(
+                f"Error loading results file '{path}': missing key {exc}")
+
+        if machine_name is not None and obj.params.get('machine') != machine_name:
+            raise util.UserError(
+                f"Error loading results file '{path}': machine name is not '{machine_name}'")
+
+        return obj
+
+    def rm(self, result_dir):
+        if self._filename is None:
+            raise ValueError("Cannot remove unnamed Results")
+
+        path = os.path.join(result_dir, self._filename)
+        os.remove(path)
+
+    @classmethod
+    def update(cls, path):
+        util.update_json(cls, path, cls.api_version, compact=True)
+
+    @property
+    def env_name(self):
+        return self._env_name
+
+    #
+    # Old data format support
+    #
+
+    @classmethod
+    def update_to_2(cls, d):
+        """
+        Reformat data in api_version 1 format to version 2.
+        """
+        try:
+            d2 = {}
+
+            d2['commit_hash'] = d['commit_hash']
+            d2['date'] = d['date']
+            d2['env_name'] = d.get('env_name',
+                                   environment.get_env_name('',
+                                                            d['python'],
+                                                            d['requirements'],
+                                                            {}))
+            d2['params'] = d['params']
+            d2['python'] = d['python']
+            d2['requirements'] = d['requirements']
+            d2['env_vars'] = d.get('env_vars', {})
+
+            # Backward-compatible load
+
+            results = {}
+            samples = {}
+            stats = {}
+            benchmark_params = {}
+
+            for key, value in d['results'].items():
                 # Backward compatibility
                 if not isinstance(value, dict):
                     value = {'result': [value], 'samples': None,
@@ -662,44 +811,82 @@ class Results(object):
                 value.setdefault('params', [])
 
                 # Assign results
-                obj._results[key] = value['result']
-                obj._samples[key] = value['samples']
-                obj._stats[key] = value['stats']
-                obj._benchmark_params[key] = value['params']
+                results[key] = value['result']
+                samples[key] = value['samples']
+                stats[key] = value['stats']
+                benchmark_params[key] = value['params']
 
             if 'profiles' in d:
-                obj._profiles = d['profiles']
-            obj._filename = os.path.join(*path.split(os.path.sep)[-2:])
+                profiles = d['profiles']
+            else:
+                profiles = {}
 
-            obj._started_at = d.get('started_at', {})
-            obj._ended_at = d.get('ended_at', {})
-            obj._benchmark_version = d.get('benchmark_version', {})
+            started_at = d.get('started_at', {})
+            duration = d.get('duration', {})
+            benchmark_version = d.get('benchmark_version', {})
+
+            # Convert to new format
+            getters = [
+                ('result', results, None),
+                ('params', benchmark_params, None),
+                ('version', benchmark_version, None),
+                ('started_at', started_at, None),
+                ('duration', duration, None),
+                ('stats_ci_99_a', stats, lambda z: z['ci_99'][0]),
+                ('stats_ci_99_b', stats, lambda z: z['ci_99'][1]),
+                ('stats_q_25', stats, lambda z: z.get('q_25')),
+                ('stats_q_75', stats, lambda z: z.get('q_75')),
+                ('stats_number', stats, lambda z: z.get('number')),
+                ('stats_repeat', stats, lambda z: z.get('repeat')),
+                ('samples', samples, None),
+                ('profile', profiles, None),
+            ]
+
+            names = set()
+            for key_dict in (results, benchmark_params):
+                names.update(key_dict.keys())
+
+            d2['result_columns'] = [x[0] for x in getters]
+            d2['results'] = {}
+
+            for name in sorted(names):
+                r = []
+
+                for key_name, key_dict, key_getter in getters:
+                    value = key_dict.get(name)
+                    if key_getter is not None and value is not None:
+                        if isinstance(value, list):
+                            value = [key_getter(z) if z is not None else None
+                                     for z in value]
+                        else:
+                            value = key_getter(value)
+
+                    if key_name.startswith('stats_') or key_name == 'duration':
+                        value = util.truncate_float_list(value)
+
+                    if key_name == 'params' and value is None:
+                        value = []
+
+                    if key_name != 'params' and isinstance(value, list):
+                        if all(x is None for x in value):
+                            value = None
+
+                    r.append(value)
+
+                while r and r[-1] is None:
+                    r.pop()
+
+                d2['results'][name] = r
+
+            d2['durations'] = {}
+            for key, value in duration.items():
+                if key.startswith('<'):
+                    d2['durations'][key] = value
+
+            return d2
         except KeyError as exc:
             raise util.UserError(
-                "Error loading results file '{0}': missing key {1}".format(
-                    path, six.text_type(exc)))
-
-        if machine_name is not None and obj.params.get('machine') != machine_name:
-            raise util.UserError(
-                "Error loading results file '{0}': machine name is not '{1}'".format(
-                    path, machine_name))
-
-        return obj
-
-    def rm(self, result_dir):
-        if self._filename is None:
-            raise ValueError("Cannot remove unnamed Results")
-
-        path = os.path.join(result_dir, self._filename)
-        os.remove(path)
-
-    @classmethod
-    def update(cls, path):
-        util.update_json(cls, path, cls.api_version)
-
-    @property
-    def env_name(self):
-        return self._env_name
+                f"Error loading results data: missing key {exc}")
 
 
 def format_benchmark_result(results, benchmark):
@@ -737,7 +924,7 @@ def format_benchmark_result(results, benchmark):
         if failure_count == total_count:
             info = "failed"
         else:
-            info = "{0}/{1} failed".format(failure_count, total_count)
+            info = f"{failure_count}/{total_count} failed"
 
     # Display results
     if benchmark['params']:
@@ -745,11 +932,15 @@ def format_benchmark_result(results, benchmark):
         if failure_count == 0:
             info = "ok"
 
-        display_result = [(v, statistics.get_err(v, s) if s is not None else None)
+        display_result = [(v, get_err(v, s) if s is not None else None)
                           for v, s in zip(result, stats)]
         display = _format_benchmark_result(display_result, benchmark)
         display = "\n".join(display).strip()
         details = display
+
+        # Display error if stderr is present
+        if failure_count > 0 and bool(results.stderr):
+            details += f"\n{results.stderr[name]}"
     else:
         if failure_count == 0:
             # Failure already shown above
@@ -757,7 +948,7 @@ def format_benchmark_result(results, benchmark):
                 display = "[]"
             else:
                 if stats[0]:
-                    err = statistics.get_err(result[0], stats[0])
+                    err = get_err(result[0], stats[0])
                 else:
                     err = None
                 display = util.human_value(result[0], benchmark['unit'], err=err)
@@ -801,7 +992,7 @@ def _format_benchmark_result(result, benchmark, max_width=None):
 
         for j, values in enumerate(itertools.product(*row_params)):
             row_results = [util.human_value(x[0], benchmark['unit'], err=x[1])
-                           for x in result[j*column_items:(j+1)*column_items]]
+                           for x in result[j * column_items:(j + 1) * column_items]]
             row = [_format_param_value(value) for value in values] + row_results
             rows.append(row)
 
@@ -816,7 +1007,7 @@ def _format_benchmark_result(result, benchmark, max_width=None):
 
     # Determine how many parameters can be fit to columns
     if max_width is None:
-        max_width = util.get_terminal_width() * 3//4
+        max_width = util.terminal_width * 3 // 4
 
     text = do_formatting(0)
     for j in range(1, len(benchmark['params'])):
