@@ -1,5 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
 r"""Regression detection in ASV is based on detecting stepwise changes
 in the graphs. The assumptions on the data are as follows: the curves
 are piecewise constant plus random noise. We don't know the scaling of
@@ -30,7 +29,8 @@ piecewise weighted fitting problem
 .. math::
    :label: gamma-opt
 
-   \mathop{\mathrm{argmin}}_{k,\{j\},\{\mu\}} \gamma k + \sum_{r=1}^k\sum_{i=j_{r-1}}^{j_r} w_i |y_i - \mu_r|
+   \mathop{\mathrm{argmin}}_{k,\{j\},\{\mu\}} \gamma k +
+   \sum_{r=1}^k\sum_{i=j_{r-1}}^{j_r} w_i |y_i - \mu_r|
 
 The differences are: as we do not need exact solutions, we add
 additional heuristics to work around the :math:`{\mathcal O}(n^2)`
@@ -121,7 +121,8 @@ Change in the exponent when :math:`\mu` is perturbed is
 .. math::
    :label:
 
-   \Delta = -\sigma^{-1}\sum_{r=1}^k \sum_{i=j_{r-1}+1}^{j_r}w_i[|y_i-\mu^*_r - \delta\mu_r| - |y_i-\mu^*_r|]
+   \Delta = -\sigma^{-1}\sum_{r=1}^k
+            \sum_{i=j_{r-1}+1}^{j_r}w_i[|y_i-\mu^*_r - \delta\mu_r| - |y_i-\mu^*_r|]
 
 Note that :math:`\sum_{i=j_{r-1}+1}^{j_r}
 w_i\mathrm{sgn}(y_i-\mu^*_r)=0`, so that response to small variations
@@ -134,7 +135,8 @@ w_i\mathrm{sgn}(y_i-\mu^*_r)=0`, so that response to small variations
 
 where :math:`N_r(\delta\mu)=\sum_{i} w_i s_i` where :math:`s_i = \pm1` depending on whether
 :math:`y_i` is above or below the perturbed median. Let us assume that in a typical case,
-:math:`N_r(\delta\mu)\sim{}m_r\bar{W}_r^2\delta\mu/\sigma` where :math:`\bar{W}_r = \frac{1}{m_r}\sum_i w_i` is
+:math:`N_r(\delta\mu)\sim{}m_r\bar{W}_r^2\delta\mu/\sigma`
+where :math:`\bar{W}_r = \frac{1}{m_r}\sum_i w_i` is
 the average weight of the interval and :math:`m_r` the number of points in the interval.
 This recovers a result we would
 have obtained in the gaussian noise case
@@ -183,7 +185,7 @@ where we neglect terms that don't affect asymptotics for
 both :math:`m, k`. The result is of course the Schwarz criterion for
 :math:`k` free model parameters. We can suspect that the factor
 :math:`k/2` should be replaced by a different number, since we have
-:math:`2k` parameters. If also the other integrals/sums can be 
+:math:`2k` parameters. If also the other integrals/sums can be
 approximated in the same way as the :math:`\{\mu\}` ones, we should
 obtain the missing part.
 
@@ -213,7 +215,8 @@ The final fitting problem then becomes
 .. math::
    :label: bic-form
 
-   \mathop{\mathrm{argmin}}_{k,\{j\},\{\mu\}} r(m) k + \ln\sum_{r=1}^k\sum_{i=j_{r-1}}^{j_r} w_i |y_i - \mu_r|
+   \mathop{\mathrm{argmin}}_{k,\{j\},\{\mu\}} r(m) k +
+   \ln\sum_{r=1}^k\sum_{i=j_{r-1}}^{j_r} w_i |y_i - \mu_r|
 
 with :math:`r(m) = \frac{\ln m}{2m}`.
 Note that it is invariant vs. rescaling of weights :math:`w_i\mapsto{}\alpha{}w_i`,
@@ -325,7 +328,8 @@ does depend on :math:`\rho`, so that the problem becomes:
 .. math::
    :label: bic-form-autocorr
 
-   \mathop{\mathrm{argmin}}_{k,\rho,\{j\},\{\mu\}} r(m) k + \ln\sum_{r=1}^k\sum_{i=j_{r-1}}^{j_r} |\epsilon_{i,r} - \rho\epsilon_{i-1,r}|
+   \mathop{\mathrm{argmin}}_{k,\rho,\{j\},\{\mu\}} r(m) k +
+   \ln\sum_{r=1}^k\sum_{i=j_{r-1}}^{j_r} |\epsilon_{i,r} - \rho\epsilon_{i-1,r}|
 
 To save computation time, we do not solve this optimization problem
 exactly. Instead, we again minimize along the :math:`\mu_r^*(\gamma)`,
@@ -353,12 +357,11 @@ currently used to provide relative data weighting (see above).
 
 """
 
-from __future__ import absolute_import, division, unicode_literals, print_function
 
 import math
 import collections
 import heapq
-import six
+from statistics import median
 
 try:
     from . import _rangemedian
@@ -387,8 +390,8 @@ def detect_steps(y, w=None):
     steps : list of (left_pos, right_pos, value, min_value, err_est)
         List containing a decomposition of the input data to a piecewise
         constant function. Each element contains the left (inclusive) and
-        right (exclusive) bounds of a segment, the average value on 
-        the segment, the minimum value in the segment, and the l1 error 
+        right (exclusive) bounds of a segment, the average value on
+        the segment, the minimum value in the segment, and the l1 error
         estimate, <|Y - avg|>. Missing data points are not necessarily
         contained in any segment; right_pos-1 is the last non-missing data
         point.
@@ -409,7 +412,7 @@ def detect_steps(y, w=None):
 
     # Weights
     if w is None:
-        w_filtered = [1]*len(y_filtered)
+        w_filtered = [1] * len(y_filtered)
     else:
         # Fill-in and normalize weights
         w_valid = [ww for ww in w if ww is not None and ww == ww]
@@ -420,7 +423,7 @@ def detect_steps(y, w=None):
         else:
             w_median = 1.0
 
-        w_filtered = [1.0]*len(y_filtered)
+        w_filtered = [1.0] * len(y_filtered)
         for j in range(len(w_filtered)):
             jj = index_map[j]
             if w[jj] is not None and w[jj] == w[jj]:
@@ -433,10 +436,11 @@ def detect_steps(y, w=None):
     steps = []
     l = 0
     for r, v, d in zip(right, values, dists):
-        steps.append((index_map[l], index_map[r-1] + 1,
-                          v,
-                          min(y_filtered[l:r]),
-                          abs(d/(r - l))))
+        steps.append((index_map[l],
+                     index_map[r - 1] + 1,
+                     v,
+                     min(y_filtered[l:r]),
+                     abs(d / (r - l))))
         l = r
     return steps
 
@@ -578,8 +582,6 @@ def solve_potts(y, w, gamma, min_size=1, max_size=None,
 
     """
 
-    inf = float('inf')
-
     if len(y) == 0:
         return [], [], []
 
@@ -601,7 +603,7 @@ def solve_potts(y, w, gamma, min_size=1, max_size=None,
     mu, dist = mu_dist.mu, mu_dist.dist
 
     if min_size >= max_pos - min_pos:
-        return [len(y)], [mu(0,len(y)-1)], [dist(0,len(y)-1)]
+        return [len(y)], [mu(0, len(y) - 1)], [dist(0, len(y) - 1)]
 
     # Perform the Bellman recursion for the optimal partition.
     # Routine "Find best partition" in [1]
@@ -620,17 +622,17 @@ def solve_potts(y, w, gamma, min_size=1, max_size=None,
         i0 = min_pos
         i1 = max_pos
 
-        B = [-gamma]*(i1 - i0 + 1)
-        p = [0]*(i1 - i0)
+        B = [-gamma] * (i1 - i0 + 1)
+        p = [0] * (i1 - i0)
         for r in range(i0, i1):
-            B[r+1-i0] = inf
+            B[r + 1 - i0] = math.inf
             a = max(r + 1 - max_size, i0)
             b = max(r + 1 - min_size + 1, i0)
             for l in range(a, b):
-                b = B[l-i0] + gamma + dist(l, r)
-                if b <= B[r+1-i0]:
-                    B[r+1-i0] = b
-                    p[r-i0] = l - 1
+                b = B[l - i0] + gamma + dist(l, r)
+                if b <= B[r + 1 - i0]:
+                    B[r + 1 - i0] = b
+                    p[r - i0] = l - 1
 
             mu_dist.cleanup_cache()
 
@@ -679,12 +681,12 @@ def solve_potts_autogamma(y, w, beta=None, **kw):
         return [], [], [], None
 
     mu_dist = get_mu_dist(y, w)
-    mu, dist = mu_dist.mu, mu_dist.dist
+    dist = mu_dist.dist
 
     if beta is None:
         beta = 4 * math.log(n) / n
 
-    gamma_0 = dist(0, n-1)
+    gamma_0 = dist(0, n - 1)
 
     if gamma_0 == 0:
         # Zero variance
@@ -693,7 +695,7 @@ def solve_potts_autogamma(y, w, beta=None, **kw):
     best_r = [None]
     best_v = [None]
     best_d = [None]
-    best_obj = [float('inf')]
+    best_obj = [math.inf]
     best_gamma = [None]
 
     def f(x):
@@ -711,7 +713,7 @@ def solve_potts_autogamma(y, w, beta=None, **kw):
             for r, v in zip(rights, values):
                 for yv, wv in zip(y[l:r], w[l:r]):
                     E = yv - v
-                    s += abs(E - rho*E_prev) * wv
+                    s += abs(E - rho * E_prev) * wv
                     E_prev = E
                 l = r
             return s
@@ -721,7 +723,7 @@ def solve_potts_autogamma(y, w, beta=None, **kw):
 
         # Measurement noise floor
         if len(v) > 2:
-            absdiff = [abs(v[j+1] - v[j]) for j in range(len(v) - 1)]
+            absdiff = [abs(v[j + 1] - v[j]) for j in range(len(v) - 1)]
             sigma_0 = 0.1 * min(absdiff)
         else:
             absv = [abs(z) for z in v]
@@ -730,7 +732,7 @@ def solve_potts_autogamma(y, w, beta=None, **kw):
 
         # Objective function
         s = sigma_star(r, v, rho_best)
-        obj = beta*len(r) + math.log(sigma_0 + s)
+        obj = beta * len(r) + math.log(sigma_0 + s)
 
         # Done
         if obj < best_obj[0]:
@@ -743,9 +745,9 @@ def solve_potts_autogamma(y, w, beta=None, **kw):
 
     # Try to find best gamma (golden section search on log-scale); we
     # don't need an accurate value for it however
-    a = math.log(0.1/n)
+    a = math.log(0.1 / n)
     b = 0.0
-    golden_search(f, a, b, xatol=abs(a)*0.1, ftol=0, expand_bounds=True)
+    golden_search(f, a, b, xatol=abs(a) * 0.1, ftol=0, expand_bounds=True)
     return best_r[0], best_v[0], best_d[0], best_gamma[0]
 
 
@@ -769,8 +771,8 @@ def solve_potts_approx(y, w, gamma=None, min_size=1, **kw):
         kw['mu_dist'] = mu_dist
 
     if gamma is None:
-        mu, dist = mu_dist.mu, mu_dist.dist
-        gamma = 3 * dist(0,n-1) * math.log(n) / n
+        dist = mu_dist.dist
+        gamma = 3 * dist(0, n - 1) * math.log(n) / n
 
     if min_size < 10:
         max_size = 20
@@ -803,11 +805,12 @@ def merge_pieces(gamma, right, values, dists, mu_dist, max_size):
 
             # Check whether merging consecutive intervals results to
             # decrease in the cost function
-            change = dist(l, right[j]-1) - (dist(l, right[j-1]-1) + dist(right[j-1], right[j]-1) + gamma)
+            change = (dist(l, right[j] - 1) -
+                      (dist(l, right[j - 1] - 1) + dist(right[j - 1], right[j] - 1) + gamma))
             if change <= min_change:
                 min_change = change
-                min_change_j = j-1
-            l = right[j-1]
+                min_change_j = j - 1
+            l = right[j - 1]
 
         if min_change_j < len(right):
             del right[min_change_j]
@@ -820,34 +823,34 @@ def merge_pieces(gamma, right, values, dists, mu_dist, max_size):
     # restriction.
     l = 0
     for j in range(1, len(right)):
-        prev_score = dist(l, right[j-1]-1) + dist(right[j-1], right[j]-1)
+        prev_score = dist(l, right[j - 1] - 1) + dist(right[j - 1], right[j] - 1)
         new_off = 0
-        for off in range(-max_size, max_size+1):
-            if right[j-1] + off - 1 < l or right[j-1] + off > right[j] - 1 or off == 0:
+        for off in range(-max_size, max_size + 1):
+            if right[j - 1] + off - 1 < l or right[j - 1] + off > right[j] - 1 or off == 0:
                 continue
-            new_score = dist(l, right[j-1]+off-1) + dist(right[j-1]+off, right[j]-1)
+            new_score = dist(l, right[j - 1] + off - 1) + dist(right[j - 1] + off, right[j] - 1)
             if new_score < prev_score:
                 new_off = off
                 prev_score = new_score
 
         if new_off != 0:
-            right[j-1] += new_off
+            right[j - 1] += new_off
 
-        l = right[j-1]
+        l = right[j - 1]
 
     # Rebuild values and dists lists
     l = 0
     values = []
     dists = []
     for j in range(len(right)):
-        dists.append(dist(l, right[j]-1))
-        values.append(mu(l, right[j]-1))
+        dists.append(dist(l, right[j] - 1))
+        values.append(mu(l, right[j] - 1))
         l = right[j]
 
     return right, values, dists
 
 
-class L1Dist(object):
+class L1Dist:
     """
     Fast computations for::
 
@@ -869,7 +872,7 @@ class L1Dist(object):
         class mu_dict(collections.defaultdict):
             def __missing__(self, a):
                 l, r = a
-                v = weighted_median(y[l:r+1], w[l:r+1])
+                v = weighted_median(y[l:r + 1], w[l:r + 1])
                 self[a] = v
                 return v
 
@@ -879,7 +882,7 @@ class L1Dist(object):
             def __missing__(self, a):
                 l, r = a
                 m = mu[l, r]
-                v = sum(wx*abs(x - m) for x, wx in zip(y[l:r+1], w[l:r+1]))
+                v = sum(wx * abs(x - m) for x, wx in zip(y[l:r + 1], w[l:r + 1]))
                 self[a] = v
                 return v
 
@@ -908,16 +911,6 @@ def get_mu_dist(y, w):
         return L1Dist(y, w)
 
 
-def median(items):
-    """Note: modifies the input list!"""
-    items.sort()
-    k = len(items)//2
-    if len(items) % 2 == 0:
-        return (items[k] + items[k - 1]) / 2
-    else:
-        return items[k]
-
-
 def rolling_median_dev(items):
     """
     Compute median(items[:j]), deviation[j]) for j in range(1, len(items))
@@ -933,7 +926,7 @@ def rolling_median_dev(items):
     try:
         while True:
             # Odd
-            v = six.next(s)
+            v = next(s)
             min_heap_sum += v
             v = -heapq.heappushpop(min_heap, -v)
             min_heap_sum -= v
@@ -944,14 +937,14 @@ def rolling_median_dev(items):
             yield (max_heap[0], d)
 
             # Even
-            v = six.next(s)
+            v = next(s)
             max_heap_sum += v
             v = heapq.heappushpop(max_heap, v)
             max_heap_sum -= v
             heapq.heappush(min_heap, -v)
             min_heap_sum += v
             d = max(0, max_heap_sum - min_heap_sum)
-            yield ((max_heap[0] - min_heap[0])/2, d)
+            yield ((max_heap[0] - min_heap[0]) / 2, d)
     except StopIteration:
         return
 
@@ -994,8 +987,8 @@ def golden_search(f, a, b, xatol=1e-6, ftol=1e-8, expand_bounds=False):
         x0 = a
         x3 = b
     else:
-        x0 = (ratio * a - (1 - ratio) * b) / (2*ratio - 1)
-        x3 = (ratio * b - (1 - ratio) * a) / (2*ratio - 1)
+        x0 = (ratio * a - (1 - ratio) * b) / (2 * ratio - 1)
+        x3 = (ratio * b - (1 - ratio) * a) / (2 * ratio - 1)
 
     x1 = ratio * x0 + (1 - ratio) * x3
     x2 = (1 - ratio) * x0 + ratio * x3
@@ -1006,7 +999,7 @@ def golden_search(f, a, b, xatol=1e-6, ftol=1e-8, expand_bounds=False):
     f0 = max(abs(f1), abs(f2))
 
     while True:
-        if abs(x0 - x3) < xatol or abs(f1 - f2) < ftol*f0:
+        if abs(x0 - x3) < xatol or abs(f1 - f2) < ftol * f0:
             break
 
         if f2 < f1:
@@ -1039,5 +1032,5 @@ def _plot_potts(x, sol):
 
     l = 0
     for r, v in zip(sol[0], sol[1]):
-        plt.plot([l, r-1], [v, v], 'b-o', hold=1)
+        plt.plot([l, r - 1], [v, v], 'b-o', hold=1)
         l = r
