@@ -1,14 +1,6 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import inspect  # Checked below.
-import sys
-
-
-class TimerawSuite(object):
+class TimerawSuite:
     def timeraw_fresh(self):
         # The test interpreter should not be polluted with anything.
         return """
