@@ -1,24 +1,15 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import os
 import sys
 import shutil
-from os.path import join, dirname
-
-import pytest
-import six
 import textwrap
+from os.path import join, dirname
 from hashlib import sha256
 
-from asv import benchmarks
-from asv import benchmark
-from asv import config
-from asv import environment
-from asv import util
+import pytest
+
+from asv import benchmarks, config, environment, util
 from asv.repo import get_repo
 
 from . import tools
@@ -30,35 +21,10 @@ INVALID_BENCHMARK_DIR = join(
 
 ASV_CONF_JSON = {
     'project': 'asv'
-    }
+}
 
-if hasattr(sys, 'pypy_version_info'):
-    ON_PYPY = True
+if util.ON_PYPY:
     ASV_CONF_JSON['pythons'] = ["pypy{0[0]}.{0[1]}".format(sys.version_info)]
-else:
-    ON_PYPY = False
-
-
-@pytest.fixture
-def benchmarks_fixture(tmpdir):
-    tmpdir = six.text_type(tmpdir)
-    os.chdir(tmpdir)
-
-    shutil.copytree(BENCHMARK_DIR, 'benchmark')
-
-    d = {}
-    d.update(ASV_CONF_JSON)
-    d['env_dir'] = "env"
-    d['benchmark_dir'] = 'benchmark'
-    d['repo'] = tools.generate_test_repo(tmpdir, [0]).path
-    d['branches'] = ["master"]
-    conf = config.Config.from_json(d)
-
-    repo = get_repo(conf)
-    envs = list(environment.get_environments(conf, None))
-    commit_hash = repo.get_hash_from_name(repo.get_branch_name())
-
-    return conf, repo, envs, commit_hash
 
 
 def test_discover_benchmarks(benchmarks_fixture):
@@ -69,19 +35,23 @@ def test_discover_benchmarks(benchmarks_fixture):
     assert len(b) == 6
 
     old_branches = conf.branches
-    conf.branches = ["master", "some-missing-branch"]  # missing branches ignored
+    conf.branches = [f"{util.git_default_branch()}",
+                     "some-missing-branch"]  # missing branches ignored
     b = benchmarks.Benchmarks.discover(conf, repo, envs, [commit_hash],
                                        regex='example')
     conf.branches = old_branches
-    assert len(b) == 36
+    if util.ON_PYPY:
+        assert len(b) == 34
+    else:
+        assert len(b) == 36
 
     b = benchmarks.Benchmarks.discover(conf, repo, envs, [commit_hash],
-                              regex='time_example_benchmark_1')
+                                       regex='time_example_benchmark_1')
     assert len(b) == 2
 
     b = benchmarks.Benchmarks.discover(conf, repo, envs, [commit_hash],
-                              regex=['time_example_benchmark_1',
-                                     'some regexp that does not match anything'])
+                                       regex=['time_example_benchmark_1',
+                                       'some regexp that does not match anything'])
     assert len(b) == 2
 
     b = benchmarks.Benchmarks.discover(conf, repo, envs, [commit_hash], regex='custom')
@@ -106,7 +76,10 @@ def test_discover_benchmarks(benchmarks_fixture):
     assert b._benchmark_selection['params_examples.track_param_selection'] == [0, 1, 2, 3]
 
     b = benchmarks.Benchmarks.discover(conf, repo, envs, [commit_hash])
-    assert len(b) == 50
+    if not util.ON_PYPY:
+        assert len(b) == 50
+    else:
+        assert len(b) == 48
 
     assert 'named.OtherSuite.track_some_func' in b
 
@@ -124,7 +97,7 @@ def test_discover_benchmarks(benchmarks_fixture):
 
 
 def test_invalid_benchmark_tree(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     os.chdir(tmpdir)
 
     d = {}
@@ -139,14 +112,14 @@ def test_invalid_benchmark_tree(tmpdir):
     commit_hash = repo.get_hash_from_name(repo.get_branch_name())
 
     with pytest.raises(util.UserError):
-        b = benchmarks.Benchmarks.discover(conf, repo, envs, [commit_hash])
+        benchmarks.Benchmarks.discover(conf, repo, envs, [commit_hash])
 
 
 def test_find_benchmarks_cwd_imports(tmpdir):
     # Test that files in the directory above the benchmark suite are
     # not importable
 
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     os.chdir(tmpdir)
 
     os.makedirs('benchmark')
@@ -187,7 +160,7 @@ def track_this():
 def test_import_failure_retry(tmpdir):
     # Test that a different commit is tried on import failure
 
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     os.chdir(tmpdir)
 
     os.makedirs('benchmark')
@@ -225,7 +198,7 @@ def test_import_failure_retry(tmpdir):
 def test_conf_inside_benchmarks_dir(tmpdir):
     # Test that the configuration file can be inside the benchmark suite
 
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     benchmark_dir = os.path.join(tmpdir, 'benchmark')
 
     os.makedirs(benchmark_dir)
@@ -256,7 +229,7 @@ def test_conf_inside_benchmarks_dir(tmpdir):
 
 
 def test_code_extraction(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     os.chdir(tmpdir)
 
     shutil.copytree(BENCHMARK_DIR, 'benchmark')
