@@ -1,25 +1,16 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import os
 import sys
-import six
-import pytest
 import json
-from collections import defaultdict
 
-from asv import config
-from asv import environment
-from asv import util
+import pytest
+
+from asv import config, environment, util
 from asv.repo import get_repo
 from asv.util import shlex_quote as quote
 
-from .tools import (PYTHON_VER1, PYTHON_VER2, DUMMY1_VERSION, DUMMY2_VERSIONS,
-                    WIN, HAS_PYPY, HAS_CONDA, HAS_VIRTUALENV, HAS_PYTHON_VER2,
-                    generate_test_repo, dummy_packages)
+from .tools import (PYTHON_VER1, PYTHON_VER2, DUMMY1_VERSION, DUMMY2_VERSIONS, WIN, HAS_PYPY,
+                    HAS_CONDA, HAS_VIRTUALENV, HAS_PYTHON_VER2, generate_test_repo)
 
 
 @pytest.mark.skipif(not (HAS_PYTHON_VER2 or HAS_CONDA),
@@ -27,7 +18,7 @@ from .tools import (PYTHON_VER1, PYTHON_VER2, DUMMY1_VERSION, DUMMY2_VERSIONS,
 def test_matrix_environments(tmpdir, dummy_packages):
     conf = config.Config()
 
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
 
     conf.pythons = [PYTHON_VER1, PYTHON_VER2]
     conf.matrix = {
@@ -47,13 +38,15 @@ def test_matrix_environments(tmpdir, dummy_packages):
             ['-c', 'import asv_dummy_test_package_1 as p, sys; sys.stdout.write(p.__version__)'],
             valid_return_codes=None)
         if 'asv_dummy_test_package_1' in env._requirements:
-            assert output.startswith(six.text_type(env._requirements['asv_dummy_test_package_1']))
+            assert output.startswith(str(env._requirements['asv_dummy_test_package_1']))
 
         output = env.run(
             ['-c', 'import asv_dummy_test_package_2 as p, sys; sys.stdout.write(p.__version__)'])
-        assert output.startswith(six.text_type(env._requirements['asv_dummy_test_package_2']))
+        assert output.startswith(str(env._requirements['asv_dummy_test_package_2']))
 
 
+@pytest.mark.skipif((not HAS_CONDA),
+                    reason="Requires conda and conda-build")
 def test_large_environment_matrix(tmpdir):
     # As seen in issue #169, conda can't handle using really long
     # directory names in its environment.  This creates an environment
@@ -61,10 +54,10 @@ def test_large_environment_matrix(tmpdir):
 
     conf = config.Config()
 
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
     conf.pythons = [PYTHON_VER1]
     for i in range(25):
-        conf.matrix['foo{0}'.format(i)] = []
+        conf.matrix[f'foo{i}'] = []
 
     environments = list(environment.get_environments(conf, None))
 
@@ -80,6 +73,7 @@ def test_large_environment_matrix(tmpdir):
         env.create()
 
 
+@pytest.mark.skipif((not HAS_CONDA), reason="Requires conda and conda-build")
 def test_presence_checks(tmpdir, monkeypatch):
     conf = config.Config()
 
@@ -89,7 +83,7 @@ def test_presence_checks(tmpdir, monkeypatch):
         # trying to cleanup environments during this test
         monkeypatch.setenv(str('CONDA_ALWAYS_COPY'), str('True'))
 
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
 
     conf.pythons = [PYTHON_VER1]
     conf.matrix = {}
@@ -194,7 +188,8 @@ def test_matrix_expand_include():
     with pytest.raises(util.UserError):
         list(environment.iter_matrix(conf.environment_type, conf.pythons, conf))
 
-
+@pytest.mark.skipif(not (HAS_PYTHON_VER2 or HAS_CONDA),
+                    reason="Requires two usable Python versions")
 def test_matrix_expand_include_detect_env_type():
     conf = config.Config()
     conf.environment_type = None
@@ -316,7 +311,8 @@ def test_iter_env_matrix_combinations():
                     for item in expected]
         for m in expected:
             m['python', None] = "2.6"
-        result = _sorted_dict_list(environment.iter_matrix(conf.environment_type, conf.pythons, conf))
+        result = _sorted_dict_list(environment.iter_matrix(conf.environment_type,
+                                                           conf.pythons, conf))
         assert result == _sorted_dict_list(expected)
 
 
@@ -325,12 +321,12 @@ def test_conda_pip_install(tmpdir, dummy_packages):
     # test that we can install with pip into a conda environment.
     conf = config.Config()
 
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
 
     conf.environment_type = "conda"
     conf.pythons = [PYTHON_VER1]
     conf.matrix = {
-        "pip+asv_dummy_test_package_2": [DUMMY2_VERSIONS[0]]
+        "pip+asv_dummy_test_package_2": DUMMY2_VERSIONS[0]
     }
     environments = list(environment.get_environments(conf, None))
 
@@ -341,18 +337,18 @@ def test_conda_pip_install(tmpdir, dummy_packages):
 
         output = env.run(
             ['-c', 'import asv_dummy_test_package_2 as p, sys; sys.stdout.write(p.__version__)'])
-        assert output.startswith(six.text_type(env._requirements['pip+asv_dummy_test_package_2']))
+        assert output.startswith(DUMMY2_VERSIONS[0])
 
 
 @pytest.mark.skipif((not HAS_CONDA), reason="Requires conda and conda-build")
 def test_conda_environment_file(tmpdir, dummy_packages):
-    env_file_name = six.text_type(tmpdir.join("environment.yml"))
+    env_file_name = str(tmpdir.join("environment.yml"))
     with open(env_file_name, "w") as temp_environment_file:
-        temp_environment_file.write(
-                'name: test_conda_envs\ndependencies:\n  - asv_dummy_test_package_2')
+        temp_environment_file.write('name: test_conda_envs\ndependencies:'
+                                    '\n  - asv_dummy_test_package_2')
 
     conf = config.Config()
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
     conf.environment_type = "conda"
     conf.pythons = [PYTHON_VER1]
     conf.conda_environment_file = env_file_name
@@ -369,11 +365,11 @@ def test_conda_environment_file(tmpdir, dummy_packages):
 
         output = env.run(
             ['-c', 'import asv_dummy_test_package_1 as p, sys; sys.stdout.write(p.__version__)'])
-        assert output.startswith(six.text_type(DUMMY1_VERSION))
+        assert output.startswith(str(DUMMY1_VERSION))
 
         output = env.run(
             ['-c', 'import asv_dummy_test_package_2 as p, sys; sys.stdout.write(p.__version__)'])
-        assert output.startswith(six.text_type(DUMMY2_VERSIONS[1]))
+        assert output.startswith(str(DUMMY2_VERSIONS[1]))
 
 
 @pytest.mark.skipif((not HAS_CONDA), reason="Requires conda and conda-build")
@@ -381,7 +377,7 @@ def test_conda_run_executable(tmpdir):
     # test that we can install with pip into a conda environment.
     conf = config.Config()
 
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
 
     conf.environment_type = "conda"
     conf.pythons = [PYTHON_VER1]
@@ -395,6 +391,8 @@ def test_conda_run_executable(tmpdir):
         env.run_executable('conda', ['info'])
 
 
+@pytest.mark.skipif(not (HAS_PYTHON_VER2 or HAS_CONDA),
+                    reason="Requires two usable Python versions")
 def test_environment_select():
     conf = config.Config()
     conf.environment_type = "conda"
@@ -423,7 +421,9 @@ def test_environment_select():
         assert items == [('conda', '1.9'), ('conda', PYTHON_VER1), ('virtualenv', PYTHON_VER1)]
 
         # Check specific python specifiers
-        environments = list(environment.get_environments(conf, ["conda:3.5", "virtualenv:"+PYTHON_VER1]))
+        environments = list(environment.get_environments(conf,
+                                                         ["conda:3.5",
+                                                          "virtualenv:" + PYTHON_VER1]))
         items = sorted((env.tool_name, env.python) for env in environments)
         assert items == [('conda', '3.5'), ('virtualenv', PYTHON_VER1)]
 
@@ -441,7 +441,9 @@ def test_environment_select():
     for env in environments:
         assert env.tool_name == "existing"
         assert env.python == "{0[0]}.{0[1]}".format(sys.version_info)
-        assert os.path.normcase(os.path.abspath(env._executable)) == os.path.normcase(os.path.abspath(sys.executable))
+        assert os.path.normcase(
+            os.path.abspath(env._executable)
+        ) == os.path.normcase(os.path.abspath(sys.executable))
 
     # Select by environment name
     conf.pythons = ["2.7"]
@@ -461,6 +463,8 @@ def test_environment_select():
     assert len(environments) == 1
 
 
+@pytest.mark.skipif(not (HAS_PYTHON_VER2 or HAS_CONDA),
+                    reason="Requires two usable Python versions")
 def test_environment_select_autodetect():
     conf = config.Config()
     conf.environment_type = "conda"
@@ -488,7 +492,7 @@ def test_environment_select_autodetect():
     environments = list(environment.get_environments(conf, ["conda:" + PYTHON_VER1]))
     assert len(environments) == 1
 
-
+@pytest.mark.skipif((not HAS_CONDA), reason="Requires conda")
 def test_matrix_empty():
     conf = config.Config()
     conf.environment_type = ""
@@ -501,6 +505,7 @@ def test_matrix_empty():
     assert items == [PYTHON_VER1]
 
 
+@pytest.mark.skipif((not HAS_CONDA), reason="Requires conda")
 def test_matrix_existing():
     conf = config.Config()
     conf.environment_type = "existing"
@@ -525,7 +530,7 @@ def test_matrix_existing():
 @pytest.mark.parametrize("channel_list,expected_channel", [
     (["defaults", "conda-forge"], "pkgs/main"),
     (["conda-forge", "defaults"], "conda-forge"),
-    ])
+])
 def test_conda_channel_addition(tmpdir,
                                 channel_list,
                                 expected_channel):
@@ -533,7 +538,7 @@ def test_conda_channel_addition(tmpdir,
     # and that we respect the specified priority order
     # of channels
     conf = config.Config()
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
     conf.environment_type = "conda"
     conf.pythons = [PYTHON_VER1]
     conf.matrix = {}
@@ -556,11 +561,11 @@ def test_conda_channel_addition(tmpdir,
         # which we are intentionally trying not to modify)
         conda = util.which('conda')
         print("\n**conda being used:", conda)
-        out_str = six.text_type(util.check_output([conda,
-                                                    'list',
-                                                    '-p',
-                                                    os.path.normpath(env._path),
-                                                    '--json']))
+        out_str = str(util.check_output([conda,
+                                        'list',
+                                         '-p',
+                                         os.path.normpath(env._path),
+                                         '--json']))
         json_package_list = json.loads(out_str)
         print(json_package_list)
         for installed_package in json_package_list:
@@ -570,12 +575,13 @@ def test_conda_channel_addition(tmpdir,
             print(installed_package)
             assert installed_package['channel'] == expected_channel
 
+
 @pytest.mark.skipif(not (HAS_PYPY and HAS_VIRTUALENV), reason="Requires pypy and virtualenv")
 def test_pypy_virtualenv(tmpdir):
     # test that we can setup a pypy environment
     conf = config.Config()
 
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
 
     conf.environment_type = "virtualenv"
     conf.pythons = ["pypy"]
@@ -587,9 +593,10 @@ def test_pypy_virtualenv(tmpdir):
     for env in environments:
         env.create()
         output = env.run(['-c', 'import sys; print(sys.pypy_version_info)'])
-        assert output.startswith(six.text_type("(major="))
+        assert "(major=" in output
 
 
+@pytest.mark.skipif((not HAS_CONDA), reason="Requires conda")
 def test_environment_name_sanitization():
     conf = config.Config()
     conf.environment_type = "conda"
@@ -605,13 +612,16 @@ def test_environment_name_sanitization():
 
 
 @pytest.mark.parametrize("environment_type", [
-    pytest.param("conda", marks=pytest.mark.skipif(not HAS_CONDA, reason="needs conda and conda-build")),
-    pytest.param("virtualenv", marks=pytest.mark.skipif(not HAS_VIRTUALENV, reason="needs virtualenv"))
+    pytest.param("conda",
+                 marks=pytest.mark.skipif(not HAS_CONDA, reason="needs conda and conda-build")),
+    pytest.param("virtualenv",
+                 marks=pytest.mark.skipif(not (HAS_PYTHON_VER2 and HAS_VIRTUALENV),
+                                          reason="needs virtualenv and python 3.7"))
 ])
 def test_environment_environ_path(environment_type, tmpdir, monkeypatch):
     # Check that virtualenv binary dirs are in the PATH
     conf = config.Config()
-    conf.env_dir = six.text_type(tmpdir.join("env"))
+    conf.env_dir = str(tmpdir.join("env"))
     conf.environment_type = environment_type
     conf.pythons = [PYTHON_VER1]
     conf.matrix = {}
@@ -637,9 +647,12 @@ def test_environment_environ_path(environment_type, tmpdir, monkeypatch):
     assert output.strip() == "Hello python path"
 
 
+
+@pytest.mark.skipif(not (HAS_PYTHON_VER2 or HAS_CONDA),
+                    reason="Requires two usable Python versions")
 def test_build_isolation(tmpdir):
     # build should not fail with build_cache on projects that have pyproject.toml
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     # Create installable repository with pyproject.toml in it
     dvcs = generate_test_repo(tmpdir, [0], dvcs_type='git')
@@ -649,7 +662,8 @@ def test_build_isolation(tmpdir):
                 'requires = ["wheel", "setuptools"]')
     dvcs.add(fn)
     dvcs.commit("Add pyproject.toml")
-    commit_hash = dvcs.get_hash("master")
+
+    commit_hash = dvcs.get_hash(f"{util.git_default_branch()}")
 
     # Setup config
     conf = config.Config()
@@ -670,7 +684,7 @@ def test_build_isolation(tmpdir):
 
 def test_custom_commands(tmpdir):
     # check custom install/uninstall/build commands work
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     dvcs = generate_test_repo(tmpdir, [0], dvcs_type='git')
 
@@ -685,9 +699,9 @@ def test_custom_commands(tmpdir):
     conf.matrix = {}
     conf.build_cache_size = 0
 
-    conf.build_command = ["python {0} {{build_cache_dir}}".format(quote(build_py))]
-    conf.install_command = ["python {0} {{env_dir}} {{build_cache_dir}}".format(quote(install_py))]
-    conf.uninstall_command = ["python {0} {{env_dir}}".format(quote(uninstall_py))]
+    conf.build_command = [f"python {quote(build_py)} {{build_cache_dir}}"]
+    conf.install_command = [f"python {quote(install_py)} {{env_dir}} {{build_cache_dir}}"]
+    conf.uninstall_command = [f"python {quote(uninstall_py)} {{env_dir}}"]
 
     with open(build_py, 'wb') as f:
         f.write(b"import os, sys\n"
@@ -758,7 +772,7 @@ def test_custom_commands(tmpdir):
 
 
 def test_installed_commit_hash(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     dvcs = generate_test_repo(tmpdir, [0], dvcs_type='git')
     commit_hash = dvcs.get_branch_hashes()[0]
@@ -779,8 +793,8 @@ def test_installed_commit_hash(tmpdir):
     env.create()
 
     # Check updating installed_commit_hash
-    assert env.installed_commit_hash == None
-    assert env._global_env_vars.get('ASV_COMMIT') == None
+    assert env.installed_commit_hash is None
+    assert env._global_env_vars.get('ASV_COMMIT') is None
     env.install_project(conf, repo, commit_hash)
     assert env.installed_commit_hash == commit_hash
     assert env._global_env_vars.get('ASV_COMMIT') == commit_hash
@@ -791,24 +805,24 @@ def test_installed_commit_hash(tmpdir):
 
     # Configuration change results to reinstall
     env._project = "something"
-    assert env.installed_commit_hash == None
+    assert env.installed_commit_hash is None
 
     # Uninstall resets hash (but not ASV_COMMIT)
     env = get_env()
     env._uninstall_project()
-    assert env.installed_commit_hash == None
-    assert env._global_env_vars.get('ASV_COMMIT') != None
+    assert env.installed_commit_hash is None
+    assert env._global_env_vars.get('ASV_COMMIT') is not None
 
     env = get_env()
-    assert env.installed_commit_hash == None
-    assert env._global_env_vars.get('ASV_COMMIT') == None
+    assert env.installed_commit_hash is None
+    assert env._global_env_vars.get('ASV_COMMIT') is None
 
 
 def test_install_success(tmpdir):
     # Check that install_project really installs the package. (gh-805)
     # This may fail if pip in install_command e.g. gets confused by an .egg-info
     # directory in its cwd to think the package is already installed.
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     dvcs = generate_test_repo(tmpdir, [0], dvcs_type='git')
     commit_hash = dvcs.get_branch_hashes()[0]
@@ -830,7 +844,7 @@ def test_install_success(tmpdir):
 
 
 def test_install_env_matrix_values(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     dvcs = generate_test_repo(tmpdir, [0], dvcs_type='git')
     commit_hash = dvcs.get_branch_hashes()[0]
@@ -916,8 +930,9 @@ def test__parse_exclude_include_rule():
     cases = [
         ({"python": "2.6", "environment_type": "conda", "sys_platform": "123",
           "env": {"A": "B"}, "env_nobuild": {"C": "D"}, "req": {"foo": "9"}},
-         {("python", None): "2.6", ("environment_type", None): "conda", ("sys_platform", None): "123",
-          ("env", "A"): "B", ("env_nobuild", "C"): "D", ("req", "foo"): "9"})
+         {("python", None): "2.6", ("environment_type", None): "conda",
+          ("sys_platform", None): "123", ("env", "A"): "B",
+          ("env_nobuild", "C"): "D", ("req", "foo"): "9"})
     ]
     for rule, expected in cases:
         parsed = environment._parse_exclude_include_rule(rule)
@@ -935,7 +950,8 @@ def test__parse_exclude_include_rule_invalid():
 
 
 def test__parse_matrix_entries():
-    entries =  {("python", None): "2.6",  ("env", "A"): "B", ("env_nobuild", "C"): "D", ("req", "foo"): "9"}
+    entries = {("python", None): "2.6", ("env", "A"): "B",
+               ("env_nobuild", "C"): "D", ("req", "foo"): "9"}
     python, requirements, tagged_env_vars = environment._parse_matrix_entries(entries)
     assert python == "2.6"
     assert requirements == {"foo": "9"}
