@@ -1,36 +1,31 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import datetime
 import os
-from os.path import abspath, dirname, join, isfile, isdir
 import shutil
-
-import six
-import pytest
+import subprocess
 import xml.etree.ElementTree as etree
+from os.path import abspath, dirname, join, isfile, isdir
+
+import json5
+import pytest
+
 try:
     import hglib
 except ImportError:
     hglib = None
 
 
-from asv import config
-from asv import util
+from asv import config, util
 from asv.repo import get_repo
 
-
 from . import tools
-from .tools import example_results
 
 BENCHMARK_DIR = abspath(join(dirname(__file__), 'benchmark'))
 
 
+@pytest.mark.skipif(tools.HAS_PYPY, reason="Flaky on pypy")
 def test_publish(tmpdir, example_results):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     os.chdir(tmpdir)
 
     result_dir = join(tmpdir, 'sample_results')
@@ -41,14 +36,15 @@ def test_publish(tmpdir, example_results):
     result_files = [fn for fn in os.listdir(join(example_results, 'cheetah'))
                     if fn.endswith('.json') and fn != 'machine.json']
     result_files.sort()
-    master_values = list(range(len(result_files)*2//3))
+    master_values = list(range(len(result_files) * 2 // 3))
     branch_values = list(range(len(master_values), len(result_files)))
     dvcs = tools.generate_test_repo(tmpdir, master_values, 'git',
-                                    [('master~6', 'some-branch', branch_values)])
+                                    [(f'{util.git_default_branch()}~6',
+                                      'some-branch', branch_values)])
 
     # Copy and modify result files, fixing commit hashes and setting result
     # dates to distinguish the two branches
-    master_commits = dvcs.get_branch_hashes('master')
+    master_commits = dvcs.get_branch_hashes(f'{util.git_default_branch()}')
     only_branch = [x for x in dvcs.get_branch_hashes('some-branch')
                    if x not in master_commits]
     commits = master_commits + only_branch
@@ -90,10 +86,10 @@ def test_publish(tmpdir, example_results):
     assert not isdir(join(tmpdir, 'html', 'graphs', 'Cython-null', 'arch-x86_64',
                           'branch-some-branch'))
     index = util.load_json(join(tmpdir, 'html', 'index.json'))
-    assert index['params']['branch'] == ['master']
+    assert index['params']['branch'] == [f'{util.git_default_branch()}']
 
     repo = get_repo(conf)
-    revision_to_hash = dict((r, h) for h, r in six.iteritems(repo.get_revisions(commits)))
+    revision_to_hash = dict((r, h) for h, r in repo.get_revisions(commits).items())
 
     def check_file(branch, cython):
         fn = join(tmpdir, 'html', 'graphs', cython, 'arch-x86_64', 'branch-' + branch,
@@ -102,7 +98,7 @@ def test_publish(tmpdir, example_results):
                   'time_coordinates.time_latitude.json')
         data = util.load_json(fn)
         data_commits = [revision_to_hash[x[0]] for x in data]
-        if branch == "master":
+        if branch == f'{util.git_default_branch()}':
             assert all(c in master_commits for c in data_commits)
         else:
             # Must contains commits from some-branch
@@ -113,21 +109,21 @@ def test_publish(tmpdir, example_results):
         # Check that revisions are strictly increasing
         assert all(x[0] < y[0] for x, y in zip(data, data[1:]))
 
-    check_file("master", "Cython")
-    check_file("master", "Cython-null")
+    check_file(f"{util.git_default_branch()}", "Cython")
+    check_file(f"{util.git_default_branch()}", "Cython-null")
 
     # Publish with branches set in the config
-    conf.branches = ['master', 'some-branch']
+    conf.branches = [f'{util.git_default_branch()}', 'some-branch']
     tools.run_asv_with_conf(conf, 'publish')
 
     # Check output
-    check_file("master", "Cython")
-    check_file("master", "Cython-null")
+    check_file(f"{util.git_default_branch()}", "Cython")
+    check_file(f"{util.git_default_branch()}", "Cython-null")
     check_file("some-branch", "Cython")
     check_file("some-branch", "Cython-null")
 
     index = util.load_json(join(tmpdir, 'html', 'index.json'))
-    assert index['params']['branch'] == ['master', 'some-branch']
+    assert index['params']['branch'] == [f'{util.git_default_branch()}', 'some-branch']
     assert index['params']['Cython'] == ['', None]
     assert index['params']['ram'] == ['8.2G', 8804682956.8]
 
@@ -139,7 +135,9 @@ def test_publish(tmpdir, example_results):
                             'os': 'Linux (Fedora 20)',
                             'python': '2.7',
                             'ram': '8.2G'}
-                            for cython in ["", None] for branch in ["master", "some-branch"]]
+                           for cython in ["",
+                                          None] for branch in [f"{util.git_default_branch()}",
+                                                               "some-branch"]]
     d = dict(expected_graph_list[0])
     d['ram'] = 8804682956.8
     expected_graph_list.append(d)
@@ -149,34 +147,12 @@ def test_publish(tmpdir, example_results):
         assert item in index['graph_param_list']
 
 
-@pytest.fixture(params=[
-    "git",
-    pytest.param("hg", marks=pytest.mark.skipif(hglib is None, reason="needs hglib")),
-])
-def generate_result_dir(request, tmpdir):
-    tmpdir = six.text_type(tmpdir)
-    dvcs_type = request.param
-
-    def _generate_result_dir(values, commits_without_result=None):
-        dvcs = tools.generate_repo_from_ops(
-            tmpdir, dvcs_type, [("commit", i) for i in range(len(values))])
-        commits = list(reversed(dvcs.get_branch_hashes()))
-        commit_values = {}
-        commits_without_result = [commits[i] for i in commits_without_result or []]
-        for commit, value in zip(commits, values):
-            if commit not in commits_without_result:
-                commit_values[commit] = value
-        conf = tools.generate_result_dir(tmpdir, dvcs, commit_values)
-        repo = get_repo(conf)
-        return conf, repo, commits
-    return _generate_result_dir
-
 def _graph_path(dvcs_type):
     if dvcs_type == "git":
-        master = "master"
+        master = f"{util.git_default_branch()}"
     elif dvcs_type == "hg":
         master = "default"
-    return join("graphs", "branch-{0}".format(master), "machine-tarzan", "time_func.json")
+    return join("graphs", f"branch-{master}", "machine-tarzan", "time_func.json")
 
 
 def test_publish_range_spec(generate_result_dir):
@@ -191,12 +167,13 @@ def test_publish_range_spec(generate_result_dir):
         assert set(data['revision_to_hash'].values()) == expected
 
 
+@pytest.mark.flaky_pypy
 def test_regression_simple(generate_result_dir):
     conf, repo, commits = generate_result_dir(5 * [1] + 5 * [10])
     tools.run_asv_with_conf(conf, "publish")
     regressions = util.load_json(join(conf.html_dir, "regressions.json"))
     expected = {"regressions": [["time_func", _graph_path(repo.dvcs), {}, None, 10.0, 1.0,
-        [[None, 5, 1.0, 10.0]]
+                [[None, 5, 1.0, 10.0]]
     ]]}
     assert regressions == expected
 
@@ -206,8 +183,8 @@ def test_regression_range(generate_result_dir):
     tools.run_asv_with_conf(conf, "publish")
     regressions = util.load_json(join(conf.html_dir, "regressions.json"))
     expected = {"regressions": [["time_func", _graph_path(repo.dvcs), {}, None, 10.0, 1.0,
-                                 [[4, 6, 1.0, 10.0]],
-    ]]}
+                                 [[4, 6, 1.0, 10.0]], ]]
+                }
     assert regressions == expected
 
 
@@ -224,7 +201,7 @@ def test_regression_double(generate_result_dir):
     tools.run_asv_with_conf(conf, "publish")
     regressions = util.load_json(join(conf.html_dir, "regressions.json"))
     expected = {"regressions": [["time_func", _graph_path(repo.dvcs), {}, None, 15.0, 1.0,
-        [[None, 5, 1.0, 10.0], [None, 10, 10.0, 15.0]],
+                [[None, 5, 1.0, 10.0], [None, 10, 10.0, 15.0]],
     ]]}
     assert regressions == expected
 
@@ -248,8 +225,8 @@ def test_regression_first_commits(generate_result_dir):
     tools.run_asv_with_conf(conf, "publish")
     regressions = util.load_json(join(conf.html_dir, "regressions.json"))
     expected = {"regressions": [["time_func", _graph_path(repo.dvcs), {}, None, 10.0, 1.0,
-                                 [[None, 5, 1.0, 10.0]]
-    ]]}
+                                 [[None, 5, 1.0, 10.0]]]]
+                }
     assert regressions == expected
 
 
@@ -280,9 +257,9 @@ def test_regression_parameterized(generate_result_dir):
     pytest.param("hg", marks=pytest.mark.skipif(hglib is None, reason="needs hglib")),
 ])
 def test_regression_multiple_branches(dvcs_type, tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     if dvcs_type == "git":
-        master = "master"
+        master = f"{util.git_default_branch()}"
     elif dvcs_type == "hg":
         master = "default"
     dvcs = tools.generate_repo_from_ops(
@@ -332,14 +309,16 @@ def test_regression_multiple_branches(dvcs_type, tmpdir):
     pytest.param("hg", marks=pytest.mark.skipif(hglib is None, reason="needs hglib"))
 ])
 def test_regression_non_monotonic(dvcs_type, tmpdir):
-    tmpdir = six.text_type(tmpdir)
-    now = datetime.datetime.now()
+    tmpdir = str(tmpdir)
+    now = datetime.datetime.now(datetime.timezone.utc)
 
-    dates = [now + datetime.timedelta(days=i) for i in range(5)] + [now - datetime.timedelta(days=i) for i in range(5)]
+    dates = [now + datetime.timedelta(days=i)
+             for i in range(5)] + [now - datetime.timedelta(days=i) for i in range(5)]
     # last commit in the past
     dates[-1] = now - datetime.timedelta(days=1)
 
-    dvcs = tools.generate_repo_from_ops(tmpdir, dvcs_type, [("commit", i, d) for i, d in enumerate(dates)])
+    dvcs = tools.generate_repo_from_ops(tmpdir, dvcs_type,
+                                        [("commit", i, d) for i, d in enumerate(dates)])
     commits = list(reversed(dvcs.get_branch_hashes()))
     commit_values = {}
     for commit, value in zip(commits, 5 * [1] + 5 * [2]):
@@ -359,16 +338,16 @@ def test_regression_threshold(generate_result_dir):
     tools.run_asv_with_conf(conf, "publish")
     regressions = util.load_json(join(conf.html_dir, "regressions.json"))
     expected = {"regressions": [["time_func", _graph_path(repo.dvcs), {}, None,
-                                  2.0, 1.0, [[None, 5, 1.0, 1.1], [None, 10, 1.1, 2.0]],
-    ]]}
+                                 2.0, 1.0, [[None, 5, 1.0, 1.1], [None, 10, 1.1, 2.0]]]]}
+
     assert regressions == expected
 
     conf.regressions_thresholds = {'.*': 0, 'time_func.*': 0.2}
     tools.run_asv_with_conf(conf, "publish")
     regressions = util.load_json(join(conf.html_dir, "regressions.json"))
     expected = {"regressions": [["time_func", _graph_path(repo.dvcs), {}, None, 2.0, 1.0,
-                                 [[None, 10, 1.1, 2.0]]
-    ]]}
+                                 [[None, 10, 1.1, 2.0]]]]}
+
     assert regressions == expected
 
 
@@ -410,8 +389,8 @@ def test_regression_atom_feed(generate_result_dir):
 def test_regression_atom_feed_update(dvcs_type, tmpdir):
     # Check that adding new commits which only change values preserves
     # feed entry ids
-    tmpdir = six.text_type(tmpdir)
-    values = 5 * [1] + 5 * [10] + 5*[15.70, 15.31]
+    tmpdir = str(tmpdir)
+    values = 5 * [1] + 5 * [10] + 5 * [15.70, 15.31]
     dvcs = tools.generate_repo_from_ops(
         tmpdir, dvcs_type, [("commit", i) for i in range(len(values))])
     commits = list(reversed(dvcs.get_branch_hashes()))
@@ -421,7 +400,10 @@ def test_regression_atom_feed_update(dvcs_type, tmpdir):
     for commit, value in zip(commits, values[:-5]):
         commit_values[commit] = value
     conf = tools.generate_result_dir(tmpdir, dvcs, commit_values,
-                                     updated=datetime.datetime(1970, 1, 1))
+                                     updated=datetime.datetime(
+                                         1970, 1, 1,
+                                         tzinfo = datetime.timezone.utc)
+                                     )
 
     tools.run_asv_with_conf(conf, "publish")
 
@@ -434,7 +416,8 @@ def test_regression_atom_feed_update(dvcs_type, tmpdir):
     shutil.rmtree(conf.results_dir)
     shutil.rmtree(conf.html_dir)
     conf = tools.generate_result_dir(tmpdir, dvcs, commit_values,
-                                     updated=datetime.datetime(1990, 1, 1))
+                                     updated=datetime.datetime(1990, 1, 1,
+                                                               tzinfo=datetime.timezone.utc))
 
     tools.run_asv_with_conf(conf, "publish")
 
@@ -457,3 +440,28 @@ def test_regression_atom_feed_update(dvcs_type, tmpdir):
         a_content = a.find('{http://www.w3.org/2005/Atom}content')
         b_content = b.find('{http://www.w3.org/2005/Atom}content')
         assert a_content.text != b_content.text
+
+def test_branch_name_is_also_filename(tmpdir):
+    # gh-1209
+    tmpdir = str(tmpdir)
+    dvcs = tools.generate_test_repo(tmpdir, list(range(10)),
+                                    dvcs_type="git",
+                                    extra_branches=[(f"{util.git_default_branch()}~4",
+                                                     "benchmarks", [11, 12, 13])])
+    subprocess.run(["asv", "quickstart", "--top-level"],
+                   cwd=dvcs.path, check=True)
+    with open(f"{dvcs.path}/asv.conf.json", 'r+') as fhandle:
+        conf = json5.load(fhandle)
+        conf["branches"] = ["benchmarks"]
+        conf["dvcs"] = "git"
+        fhandle.seek(0)
+        json5.dump(conf, fhandle, indent=4)
+        fhandle.truncate()
+    subprocess.run(["asv", "machine", "--yes"],
+                   cwd=dvcs.path, check=False)
+    subprocess.run(["asv", "run"],
+                   cwd=dvcs.path, check=False)
+    retdat = subprocess.run(["asv", "publish"], cwd=dvcs.path,
+                            capture_output=True, text=True)
+    assert "git rev-list --first-parent benchmarks" not in retdat.stderr
+    assert retdat.returncode == 0
