@@ -1,21 +1,13 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import os
-from os.path import join
-
-import six
-import pytest
 import tempfile
 import shutil
 import datetime
+from os.path import join
 
-from asv import config
-from asv import repo
-from asv import util
+import pytest
+
+from asv import config, repo, util
 
 try:
     import hglib
@@ -97,10 +89,12 @@ def _test_branches(conf, branch_commits, require_describe=False):
 
 
 def test_repo_git(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
-    dvcs = tools.generate_test_repo(tmpdir, list(range(10)), dvcs_type='git',
-                                    extra_branches=[('master~4', 'some-branch',[11, 12, 13])])
+    dvcs = tools.generate_test_repo(tmpdir, list(range(10)),
+                                    dvcs_type='git',
+                                    extra_branches=[(f'{util.git_default_branch()}~4',
+                                                     'some-branch', [11, 12, 13])])
 
     mirror_dir = join(tmpdir, "repo")
 
@@ -109,13 +103,18 @@ def test_repo_git(tmpdir):
 
         conf.project = mirror_dir
         conf.repo = dvcs.path
-        _test_generic_repo(conf, tmpdir, 'master~4..master', 'master', 'tag5',
+        _test_generic_repo(conf, tmpdir,
+                           f'{util.git_default_branch()}~4..{util.git_default_branch()}',
+                           f'{util.git_default_branch()}', 'tag5',
                            is_remote=is_remote)
 
-        conf.branches = ['master', 'some-branch']
+        conf.branches = [f'{util.git_default_branch()}', 'some-branch']
         branch_commits = {
-            'master': [dvcs.get_hash('master'), dvcs.get_hash('master~6')],
-            'some-branch': [dvcs.get_hash('some-branch'), dvcs.get_hash('some-branch~6')]
+            f'{util.git_default_branch()}':
+            [dvcs.get_hash(f'{util.git_default_branch()}'),
+             dvcs.get_hash(f'{util.git_default_branch()}~6')],
+            'some-branch': [dvcs.get_hash('some-branch'),
+                            dvcs.get_hash('some-branch~6')]
         }
         _test_branches(conf, branch_commits, require_describe=True)
 
@@ -141,7 +140,7 @@ def test_repo_git(tmpdir):
 
 
 def test_repo_git_annotated_tag_date(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     dvcs = tools.generate_test_repo(tmpdir, list(range(5)), dvcs_type='git')
 
@@ -158,12 +157,12 @@ def test_repo_git_annotated_tag_date(tmpdir):
 @pytest.mark.skipif(hglib is None,
                     reason="needs hglib")
 def test_repo_hg(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     conf = config.Config()
 
-    dvcs = tools.generate_test_repo(tmpdir, list(range(10)), dvcs_type='hg', 
-                                    extra_branches=[('default~4', 'somebranch',[11, 12, 13])])
+    dvcs = tools.generate_test_repo(tmpdir, list(range(10)), dvcs_type='hg',
+                                    extra_branches=[('default~4', 'somebranch', [11, 12, 13])])
 
     mirror_dir = join(tmpdir, "repo")
 
@@ -202,62 +201,6 @@ def test_repo_hg(tmpdir):
         Hg.url_match = old_url_match
 
 
-@pytest.fixture(params=[
-    "git",
-    pytest.param("hg", marks=pytest.mark.skipif(hglib is None, reason="needs hglib")),
-])
-def two_branch_repo_case(request, tmpdir):
-    r"""
-    This test ensure we follow the first parent in case of merges
-
-    The revision graph looks like this:
-
-        @  Revision 6 (default)
-        |
-        | o  Revision 5 (stable)
-        | |
-        | o  Merge master
-        |/|
-        o |  Revision 4
-        | |
-        o |  Merge stable
-        |\|
-        o |  Revision 3
-        | |
-        | o  Revision 2
-        |/
-        o  Revision 1
-
-    """
-    dvcs_type = request.param
-    tmpdir = six.text_type(tmpdir)
-    if dvcs_type == "git":
-        master = "master"
-    elif dvcs_type == "hg":
-        master = "default"
-    dvcs = tools.generate_repo_from_ops(tmpdir, dvcs_type, [
-        ("commit", 1),
-        ("checkout", "stable", master),
-        ("commit", 2),
-        ("checkout", master),
-        ("commit", 3),
-        ("merge", "stable"),
-        ("commit", 4),
-        ("checkout", "stable"),
-        ("merge", master, "Merge master"),
-        ("commit", 5),
-        ("checkout", master),
-        ("commit", 6),
-    ])
-
-    conf = config.Config()
-    conf.branches = [master, "stable"]
-    conf.repo = dvcs.path
-    conf.project = join(tmpdir, "repo")
-    r = repo.get_repo(conf)
-    return dvcs, master, r, conf
-
-
 def test_get_branch_commits(two_branch_repo_case):
     # Test that get_branch_commits() return an ordered list of commits (last
     # first) and follow first parent in case of merge
@@ -318,29 +261,32 @@ def test_get_new_branch_commits(two_branch_repo_case, existing, expected):
 
 
 def test_git_submodule(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     # State 0 (no submodule)
     dvcs = tools.generate_test_repo(tmpdir, values=[0], dvcs_type='git')
     sub_dvcs = tools.generate_test_repo(tmpdir, values=[0], dvcs_type='git')
     ssub_dvcs = tools.generate_test_repo(tmpdir, values=[0], dvcs_type='git')
-    commit_hash_0 = dvcs.get_hash("master")
+    commit_hash_0 = dvcs.get_hash(f"{util.git_default_branch()}")
 
     # State 1 (one submodule)
-    dvcs.run_git(['submodule', 'add', sub_dvcs.path, 'sub1'])
+    dvcs.run_git(['-c', 'protocol.file.allow=always',
+                  'submodule', 'add', sub_dvcs.path, 'sub1'])
     dvcs.commit('Add sub1')
-    commit_hash_1 = dvcs.get_hash("master")
+    commit_hash_1 = dvcs.get_hash(f"{util.git_default_branch()}")
 
     # State 2 (one submodule with sub-submodule)
-    dvcs.run_git(['submodule', 'update', '--init'])
+    dvcs.run_git(['-c', 'protocol.file.allow=always',
+                  'submodule', 'update', '--init'])
     sub1_dvcs = tools.Git(join(dvcs.path, 'sub1'))
-    sub_dvcs.run_git(['submodule', 'add', ssub_dvcs.path, 'ssub1'])
+    sub_dvcs.run_git(['-c', 'protocol.file.allow=always',
+                      'submodule', 'add', ssub_dvcs.path, 'ssub1'])
     sub_dvcs.commit('Add sub1')
     sub1_dvcs.run_git(['pull'])
     dvcs.run_git(['add', 'sub1'])
     dvcs.commit('Update sub1')
-    sub1_hash_2 = sub1_dvcs.get_hash("master")
-    commit_hash_2 = dvcs.get_hash("master")
+    sub1_hash_2 = sub1_dvcs.get_hash(f"{util.git_default_branch()}")
+    commit_hash_2 = dvcs.get_hash(f"{util.git_default_branch()}")
 
     # State 3 (one submodule; sub-submodule removed)
     sub_dvcs.run_git(['rm', '-f', 'ssub1'])
@@ -348,20 +294,18 @@ def test_git_submodule(tmpdir):
     sub1_dvcs.run_git(['pull'])
     dvcs.run_git(['add', 'sub1'])
     dvcs.commit('Update sub1 again')
-    commit_hash_3 = dvcs.get_hash("master")
+    commit_hash_3 = dvcs.get_hash(f"{util.git_default_branch()}")
 
     # State 4 (back to one submodule with sub-submodule)
     sub1_dvcs.run_git(['checkout', sub1_hash_2])
     dvcs.run_git(['add', 'sub1'])
     dvcs.commit('Update sub1 3rd time')
-    commit_hash_4 = dvcs.get_hash("master")
+    commit_hash_4 = dvcs.get_hash(f"{util.git_default_branch()}")
 
     # State 5 (remove final submodule)
     dvcs.run_git(['rm', '-f', 'sub1'])
     dvcs.commit('Remove sub1')
-    commit_hash_5 = dvcs.get_hash("master")
-
-
+    commit_hash_5 = dvcs.get_hash(f"{util.git_default_branch()}")
     # Verify clean operation
     conf = config.Config()
     conf.branches = [None]
@@ -414,7 +358,7 @@ def test_git_submodule(tmpdir):
 def test_root_ceiling(dvcs_type, tmpdir):
     # Check that git/hg does not try to look for repository in parent
     # directories.
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     dvcs1 = tools.generate_repo_from_ops(tmpdir, dvcs_type, [("commit", 1)])
     dvcs2 = tools.generate_repo_from_ops(tmpdir, dvcs_type, [("commit", 2)])
     commit1 = dvcs1.get_branch_hashes()[0]
@@ -449,7 +393,7 @@ def test_root_ceiling(dvcs_type, tmpdir):
     pytest.param("hg", marks=pytest.mark.skipif(hglib is None, reason="needs hglib"))
 ])
 def test_no_such_name_error(dvcs_type, tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     dvcs = tools.generate_test_repo(tmpdir, values=[0], dvcs_type=dvcs_type)
 
     conf = config.Config()
@@ -481,12 +425,12 @@ def test_no_such_name_error(dvcs_type, tmpdir):
     pytest.param("hg", marks=pytest.mark.skipif(hglib is None, reason="needs hglib"))
 ])
 def test_filter_date_period(tmpdir, dvcs_type):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     dates = [
-        datetime.datetime(2001, 1, 1),
-        datetime.datetime(2001, 1, 2),
-        datetime.datetime(2001, 1, 8)
+        datetime.datetime(2001, 1, 1, tzinfo=datetime.timezone.utc),
+        datetime.datetime(2001, 1, 2, tzinfo=datetime.timezone.utc),
+        datetime.datetime(2001, 1, 8, tzinfo=datetime.timezone.utc)
     ]
 
     dvcs = tools.generate_repo_from_ops(
@@ -501,12 +445,12 @@ def test_filter_date_period(tmpdir, dvcs_type):
     r = repo.get_repo(conf)
 
     # Basic filtering
-    weekly_commits = r.filter_date_period(commits, 60*60*24*7)
+    weekly_commits = r.filter_date_period(commits, 60 * 60 * 24 * 7)
     assert weekly_commits == [commits[0], commits[2]]
 
-    daily_commits = r.filter_date_period(commits, 60*60*24)
+    daily_commits = r.filter_date_period(commits, 60 * 60 * 24)
     assert daily_commits == commits
 
     # Test with old_commits specified
-    monthly_commits = r.filter_date_period(commits[1:], 60*60*24*30, commits[:1])
+    monthly_commits = r.filter_date_period(commits[1:], 60 * 60 * 24 * 30, commits[:1])
     assert monthly_commits == []
