@@ -1,46 +1,22 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import sys
 import os
-from os.path import join
-
-import pytest
+import re
 import shutil
 import glob
 import datetime
 import textwrap
+from os.path import join
 
-from asv import results
-from asv import environment
-from asv import repo
-from asv import util
+import pytest
+
+from asv import results, environment, repo, util
 from asv.commands.run import Run
+from asv.commands import make_argparser
 
 from . import tools
-from .tools import dummy_packages, WIN
-from .test_workflow import basic_conf, generate_basic_conf
-
-
-@pytest.fixture
-def basic_conf(tmpdir, dummy_packages):
-    return generate_basic_conf(tmpdir)
-
-
-@pytest.fixture
-def basic_conf_with_subdir(tmpdir, dummy_packages):
-    return generate_basic_conf(tmpdir, 'some_subdir')
-
-
-@pytest.fixture
-def existing_env_conf(tmpdir):
-    tmpdir, local, conf, machine_file = generate_basic_conf(tmpdir)
-    conf.environment_type = "existing"
-    conf.pythons = ["same"]
-    return tmpdir, local, conf, machine_file
+from .tools import WIN
 
 
 def test_set_commit_hash(capsys, existing_env_conf):
@@ -61,22 +37,22 @@ def test_set_commit_hash(capsys, existing_env_conf):
     assert times.commit_hash == commit_hash
 
 
-def test_run_spec(basic_conf):
-    tmpdir, local, conf, machine_file = basic_conf
+def test_run_spec(basic_conf_2):
+    tmpdir, local, conf, machine_file = basic_conf_2
     conf.build_cache_size = 5
 
-    extra_branches = [('master~1', 'some-branch', [12])]
+    extra_branches = [(f'{util.git_default_branch()}~1', 'some-branch', [12])]
     dvcs_path = os.path.join(tmpdir, 'test_repo2')
     dvcs = tools.generate_test_repo(dvcs_path, [1, 2],
                                     extra_branches=extra_branches)
     conf.repo = dvcs.path
 
-    initial_commit = dvcs.get_hash("master~1")
-    master_commit = dvcs.get_hash("master")
+    initial_commit = dvcs.get_hash(f"{util.git_default_branch()}~1")
+    master_commit = dvcs.get_hash(f"{util.git_default_branch()}")
     branch_commit = dvcs.get_hash("some-branch")
     template_dir = os.path.join(tmpdir, "results_workflow_template")
     results_dir = os.path.join(tmpdir, 'results_workflow')
-    tools.run_asv_with_conf(conf, 'run', initial_commit+"^!",
+    tools.run_asv_with_conf(conf, 'run', initial_commit + "^!",
                             '--bench=time_secondary.track_value',
                             '--quick',
                             _machine_file=join(tmpdir, 'asv-machine.json'))
@@ -89,8 +65,7 @@ def test_run_spec(basic_conf):
 
         args = ["run", "--quick", "--skip-existing-successful",
                 "--bench=time_secondary.track_value",
-                "-s", "1000"  # large number of steps should be noop
-               ]
+                "-s", "1000"]  # large number of steps should be noop
         if range_spec is not None:
             args.append(range_spec)
         conf.branches = branches
@@ -107,8 +82,8 @@ def test_run_spec(basic_conf):
         expected = set(['machine.json'])
         for commit in expected_commits:
             for psver in tools.DUMMY2_VERSIONS:
-                expected.add('{0}-{1}-py{2}-asv_dummy_test_package_1-asv_dummy_test_package_2{3}.json'.format(
-                    commit[:8], tool_name, pyver, psver))
+                expected.add(f'{commit[:8]}-{tool_name}-py{pyver}-asv_dummy_'
+                             f'test_package_1-asv_dummy_test_package_2{psver}.json')
 
         result_files = os.listdir(join(tmpdir, 'results_workflow', 'orangutan'))
 
@@ -122,7 +97,8 @@ def test_run_spec(basic_conf):
         (["some-branch"], [initial_commit, branch_commit]),
 
         # With two branch in config, should apply to specified branches
-        (["master", "some-branch"], [initial_commit, master_commit, branch_commit]),
+        ([f"{util.git_default_branch()}", "some-branch"],
+         [initial_commit, master_commit, branch_commit]),
     ):
         for range_spec in (None, "NEW", "ALL"):
             _test_run(range_spec, branches, expected_commits)
@@ -132,9 +108,9 @@ def test_run_spec(basic_conf):
     with open(os.path.join(tmpdir, 'hashes_to_benchmark'), 'w') as f:
         for commit in expected_commits:
             f.write(commit + "\n")
-        f.write("master~1\n")
+        f.write(f"{util.git_default_branch()}~1\n")
         f.write("some-bad-hash-that-will-be-ignored\n")
-        expected_commits += (dvcs.get_hash("master~1"),)
+        expected_commits += (dvcs.get_hash(f"{util.git_default_branch()}~1"),)
     _test_run('HASHFILE:hashes_to_benchmark', [None], expected_commits)
 
 
@@ -158,10 +134,11 @@ def test_run_build_failure(basic_conf):
     dvcs.commit("Fix setup.py")
 
     # Test running it
-    timestamp = util.datetime_to_js_timestamp(datetime.datetime.utcnow())
+    timestamp = util.datetime_to_js_timestamp(datetime.datetime.now(datetime.timezone.utc))
 
     bench_name = 'time_secondary.track_value'
-    for commit in ['master^!', 'master~1^!']:
+    for commit in [f'{util.git_default_branch()}^!',
+                   f'{util.git_default_branch()}~1^!']:
         tools.run_asv_with_conf(conf, 'run', commit,
                                 '--quick', '--show-stderr',
                                 '--bench', bench_name,
@@ -170,9 +147,9 @@ def test_run_build_failure(basic_conf):
     # Check results
     hashes = dvcs.get_branch_hashes()
     fn_broken, = glob.glob(join(tmpdir, 'results_workflow', 'orangutan',
-                                    hashes[1][:8] + '-*.json'))
+                                hashes[1][:8] + '-*.json'))
     fn_ok, = glob.glob(join(tmpdir, 'results_workflow', 'orangutan',
-                                hashes[0][:8] + '-*.json'))
+                            hashes[0][:8] + '-*.json'))
 
     data_broken = util.load_json(fn_broken)
     data_ok = util.load_json(fn_ok)
@@ -207,7 +184,7 @@ def test_run_with_repo_subdir(basic_conf_with_subdir):
     # This benchmark imports the project under test (asv_test_repo)
     bench_name = 'params_examples.track_find_test'
     # Test with a single changeset
-    tools.run_asv_with_conf(conf, 'run', 'master^!',
+    tools.run_asv_with_conf(conf, 'run', f'{util.git_default_branch()}^!',
                             '--quick', '--show-stderr',
                             '--bench', bench_name,
                             _machine_file=machine_file)
@@ -226,7 +203,7 @@ def test_benchmark_param_selection(basic_conf):
     tmpdir, local, conf, machine_file = basic_conf
     conf.matrix = {}
     tools.generate_test_repo(tmpdir, values=[(1, 2, 3)])
-    tools.run_asv_with_conf(conf, 'run', 'master^!',
+    tools.run_asv_with_conf(conf, 'run', f'{util.git_default_branch()}^!',
                             '--quick', '--show-stderr',
                             '--bench', r'track_param_selection\(.*, 3\)',
                             _machine_file=machine_file)
@@ -250,15 +227,15 @@ def test_benchmark_param_selection(basic_conf):
                             _machine_file=machine_file)
 
 
-def test_run_append_samples(basic_conf):
-    tmpdir, local, conf, machine_file = basic_conf
+def test_run_append_samples(basic_conf_2):
+    tmpdir, local, conf, machine_file = basic_conf_2
 
     # Only one environment
     conf.matrix['asv_dummy_test_package_2'] = conf.matrix['asv_dummy_test_package_2'][:1]
 
     # Tests multiple calls to "asv run --append-samples"
     def run_it():
-        tools.run_asv_with_conf(conf, 'run', "master^!",
+        tools.run_asv_with_conf(conf, 'run', f"{util.git_default_branch()}^!",
                                 '--bench', 'time_examples.TimeSuite.time_example_benchmark_1',
                                 '--append-samples', '-a', 'repeat=(1, 1, 10.0)', '-a', 'rounds=1',
                                 '-a', 'number=1', '-a', 'warmup_time=0',
@@ -271,13 +248,17 @@ def test_run_append_samples(basic_conf):
                   if fn != 'machine.json']
 
     data = util.load_json(result_fn)
-    value = dict(zip(data['result_columns'], data['results']['time_examples.TimeSuite.time_example_benchmark_1']))
+    value = dict(zip(
+        data['result_columns'],
+        data['results']['time_examples.TimeSuite.time_example_benchmark_1']))
     assert value['stats_q_25'][0] is not None
     assert len(value['samples'][0]) == 1
 
     run_it()
     data = util.load_json(result_fn)
-    value = dict(zip(data['result_columns'], data['results']['time_examples.TimeSuite.time_example_benchmark_1']))
+    value = dict(zip(
+        data['result_columns'],
+        data['results']['time_examples.TimeSuite.time_example_benchmark_1']))
     assert len(value['samples'][0]) == 2
 
 
@@ -287,13 +268,11 @@ def test_cpu_affinity(basic_conf):
     # Only one environment
     conf.matrix = {}
 
-    tools.run_asv_with_conf(conf, 'run', "master^!",
+    tools.run_asv_with_conf(conf, 'run', f"{util.git_default_branch()}^!",
                             '--bench', 'time_examples.TimeSuite.time_example_benchmark_1',
                             '--cpu-affinity=0', '-a', 'repeat=(1, 1, 10.0)', '-a', 'rounds=1',
                             '-a', 'number=1', '-a', 'warmup_time=0',
                             _machine_file=machine_file)
-
-
     # Check run produced a result
     result_dir = join(tmpdir, 'results_workflow', 'orangutan')
     result_fn, = [join(result_dir, fn) for fn in os.listdir(result_dir)
@@ -310,7 +289,7 @@ def test_env_matrix_value(basic_conf):
     def check_env_matrix(env_build, env_nobuild):
         conf.matrix = {"env": env_build, "env_nobuild": env_nobuild}
 
-        tools.run_asv_with_conf(conf, 'run', "master^!",
+        tools.run_asv_with_conf(conf, 'run', f"{util.git_default_branch()}^!",
                                 '--bench', 'time_secondary.track_environment_value',
                                 _machine_file=machine_file)
 
@@ -331,8 +310,8 @@ def test_env_matrix_value(basic_conf):
     check_env_matrix({'SOME_TEST_VAR': ['1', '2']}, {})
 
 
-def test_parallel(basic_conf, dummy_packages):
-    tmpdir, local, conf, machine_file = basic_conf
+def test_parallel(basic_conf_2, dummy_packages):
+    tmpdir, local, conf, machine_file = basic_conf_2
 
     if WIN and os.path.basename(sys.argv[0]).lower().startswith('py.test'):
         # Multiprocessing in spawn mode can result to problems with py.test
@@ -346,7 +325,7 @@ def test_parallel(basic_conf, dummy_packages):
         "env_nobuild": {"SOME_OTHER_TEST_VAR": ["1", "2"]}
     }
 
-    tools.run_asv_with_conf(conf, 'run', "master^!",
+    tools.run_asv_with_conf(conf, 'run', f"{util.git_default_branch()}^!",
                             '--bench', 'time_secondary.track_environment_value',
                             '--parallel=2', _machine_file=machine_file)
 
@@ -355,9 +334,9 @@ def test_filter_date_period(tmpdir, basic_conf):
     tmpdir, local, conf, machine_file = basic_conf
 
     dates = [
-        datetime.datetime(2001, 1, 1),
-        datetime.datetime(2001, 1, 2),
-        datetime.datetime(2001, 1, 8)
+        datetime.datetime(2001, 1, 1, tzinfo=datetime.timezone.utc),
+        datetime.datetime(2001, 1, 2, tzinfo=datetime.timezone.utc),
+        datetime.datetime(2001, 1, 8, tzinfo=datetime.timezone.utc)
     ]
 
     dvcs = tools.generate_repo_from_ops(
@@ -368,7 +347,7 @@ def test_filter_date_period(tmpdir, basic_conf):
     conf.repo = dvcs.path
     conf.matrix = {}
 
-    tools.run_asv_with_conf(conf, 'run', 'master',
+    tools.run_asv_with_conf(conf, 'run', f'{util.git_default_branch()}',
                             '--date-period=1w',
                             '--quick', '--show-stderr',
                             '--bench=time_secondary.track_value',
@@ -388,22 +367,58 @@ def test_format_durations():
     durations = {'foo': 1, 'bar': 2, 'quux': 3}
 
     msg = Run.format_durations(durations, 2)
+    # Removing tailing spaces, so they don't need to be in `expected`
+    msg = re.sub(r' *\n', r'\n', msg)
     expected = textwrap.dedent("""\
     =========== ================
-     benchmark   total duration 
+     benchmark   total duration
     ----------- ----------------
-        quux         3.00s      
-        bar          2.00s      
-        ...           ...       
-       total         6.00s      
+        quux         3.00s
+        bar          2.00s
+        ...           ...
+       total         6.00s
     =========== ================""")
     assert msg == expected
 
 
-def test_return_code_strict_mode(tmpdir, basic_conf):
-    tmpdir, local, conf, machine_file = basic_conf
+def test_return_code(tmpdir, basic_conf_2):
+    tmpdir, local, conf, machine_file = basic_conf_2
 
-    res = tools.run_asv_with_conf(conf, 'run', 'master^!', '--quick',
-                                  '--strict', '--bench', 'TimeSecondary',
+    res = tools.run_asv_with_conf(conf, 'run', f'{util.git_default_branch()}^!', '--quick',
+                                  '--bench', 'TimeSecondary',
                                   _machine_file=machine_file)
     assert res == 2
+
+
+def test_run_python_same(capsys, basic_conf):
+    tmpdir, local, conf, machine_file = basic_conf
+
+    # Test Run runs with python=same
+    tools.run_asv_with_conf(conf, 'run', '--python=same',
+                            '--bench=time_secondary.TimeSecondary.time_exception',
+                            '--bench=time_secondary.track_value',
+                            _machine_file=machine_file)
+    text, err = capsys.readouterr()
+
+    assert re.search("time_exception.*failed", text, re.S)
+    assert re.search(r"time_secondary.track_value\s+42.0", text)
+
+    # Check that it did not clone or install
+    assert "Cloning" not in text
+    assert "Installing" not in text
+
+
+def test_run_python_arg():
+    parser, subparsers = make_argparser()
+
+    argv = ['run', 'ALL']
+    args = parser.parse_args(argv)
+    assert args.env_spec == []
+
+
+def test_run_steps_arg():
+    parser, subparsers = make_argparser()
+
+    argv = ['run', '--steps=20', 'ALL']
+    args = parser.parse_args(argv)
+    assert args.steps == 20
