@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import sys
 import os
 import shutil
@@ -12,26 +8,19 @@ import pstats
 import collections
 import socket
 import json
-
-import six
-import pytest
-
 from os.path import join
 
-from asv import benchmarks
-from asv import config
-from asv import environment
-from asv import runner
-from asv import util
+import pytest
+
+from asv import benchmarks, config, environment, runner, util
 from asv.results import Results
 
-from .test_benchmarks import benchmarks_fixture, ASV_CONF_JSON, BENCHMARK_DIR
-
+from .test_benchmarks import ASV_CONF_JSON, BENCHMARK_DIR
 
 ON_PYPY = hasattr(sys, 'pypy_version_info')
 
 
-class ResultsWrapper(object):
+class ResultsWrapper:
     tuple_type = collections.namedtuple('tuple_type', ['result', 'stats', 'samples',
                                                        'params', 'stderr', 'errcode',
                                                        'profile', 'started_at', 'duration'])
@@ -65,7 +54,7 @@ class ResultsWrapper(object):
 def test_run_benchmarks(benchmarks_fixture, tmpdir):
     conf, repo, envs, commit_hash = benchmarks_fixture
 
-    start_timestamp = datetime.datetime.utcnow()
+    start_timestamp = datetime.datetime.now(datetime.timezone.utc)
 
     b = benchmarks.Benchmarks.discover(conf, repo, envs, [commit_hash])
 
@@ -85,8 +74,6 @@ def test_run_benchmarks(benchmarks_fixture, tmpdir):
         append_samples=True, record_samples=True)
     times = ResultsWrapper(results, b)
 
-    end_timestamp = datetime.datetime.utcnow()
-
     assert len(times) == len(b)
     assert times[
         'time_examples.TimeSuite.time_example_benchmark_1'].result != [None]
@@ -119,15 +106,20 @@ def test_run_benchmarks(benchmarks_fixture, tmpdir):
 
     assert times['time_examples.TimeWithBadTimer.time_it'].result == [0.0]
 
-    assert times['params_examples.track_param'].params == [["<class 'benchmark.params_examples.ClassOne'>",
-                                                               "<class 'benchmark.params_examples.ClassTwo'>"]]
+    assert times['params_examples.track_param'].params == [
+        [
+            "<class 'benchmark.params_examples.ClassOne'>",
+            "<class 'benchmark.params_examples.ClassTwo'>"
+        ]
+    ]
     assert times['params_examples.track_param'].result == [42, 42]
 
-    assert times['params_examples.mem_param'].params == [['10', '20'], ['2', '3']]
-    assert len(times['params_examples.mem_param'].result) == 2*2
+    if not util.ON_PYPY:
+        assert times['params_examples.mem_param'].params == [['10', '20'], ['2', '3']]
+        assert len(times['params_examples.mem_param'].result) == 2 * 2
 
     assert times['params_examples.ParamSuite.track_value'].params == [["'a'", "'b'", "'c'"]]
-    assert times['params_examples.ParamSuite.track_value'].result == [1+0, 2+0, 3+0]
+    assert times['params_examples.ParamSuite.track_value'].result == [1 + 0, 2 + 0, 3 + 0]
 
     assert isinstance(times['params_examples.TuningTest.time_it'].result[0], float)
     assert isinstance(times['params_examples.TuningTest.time_it'].result[1], float)
@@ -154,7 +146,7 @@ def test_run_benchmarks(benchmarks_fixture, tmpdir):
     assert times['cache_examples.time_fail_second_run'].result == [None]
     assert times['cache_examples.time_fail_second_run'].samples == [None]
 
-    profile_path = join(six.text_type(tmpdir), 'test.profile')
+    profile_path = join(str(tmpdir), 'test.profile')
     with open(profile_path, 'wb') as fd:
         fd.write(times['time_secondary.track_value'].profile)
     pstats.Stats(profile_path)
@@ -165,7 +157,7 @@ def test_run_benchmarks(benchmarks_fixture, tmpdir):
     assert times['time_examples.TimeWithRepeat.time_it'].stderr.split() == expected
 
     # Calibration of iterations should not rerun setup
-    expected = (['setup']*2, ['setup']*3)
+    expected = (['setup'] * 2, ['setup'] * 3)
     assert times['time_examples.TimeWithRepeatCalibrate.time_it'].stderr.split() in expected
 
     # Check tuple-form repeat attribute produced results
@@ -204,7 +196,7 @@ def test_skip_param_selection():
     d.update(ASV_CONF_JSON)
     conf = config.Config.from_json(d)
 
-    class DummyEnv(object):
+    class DummyEnv:
         name = 'env'
 
     d = [
@@ -218,20 +210,28 @@ def test_skip_param_selection():
     results = Results.unnamed()
     b = benchmarks.Benchmarks(conf, d, [r'test_nonparam', r'test_param\([23]\)'])
 
-    results.add_result(b['test_param'],
-                       runner.BenchmarkResult(result=[1, 2, 3], samples=[None]*3, number=[None]*3,
-                                              errcode=0, stderr='', profile=None))
+    results.add_result(
+        b['test_param'],
+        runner.BenchmarkResult(
+            result=[1, 2, 3],
+            samples=[None] * 3,
+            number=[None] * 3,
+            errcode=0,
+            stderr='',
+            profile=None
+        )
+    )
 
     runner.skip_benchmarks(b, DummyEnv(), results)
 
-    assert results._results.get('test_nonparam') == None
+    assert results._results.get('test_nonparam') is None
     assert results._results['test_param'] == [1, None, None]
 
 
 @pytest.mark.skipif(not (hasattr(os, 'fork') and hasattr(socket, 'AF_UNIX')),
                     reason="test requires fork and unix sockets")
 def test_forkserver(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     os.chdir(tmpdir)
 
     shutil.copytree(BENCHMARK_DIR, 'benchmark')
@@ -291,7 +291,7 @@ def clear_pyc(path):
 
 @needs_unix_socket_mark
 def test_forkserver_preimport(tmpdir):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
     os.chdir(tmpdir)
 
     os.makedirs('benchmark')
@@ -318,7 +318,7 @@ def test_forkserver_preimport(tmpdir):
     finally:
         spawner.close()
 
-    assert success == True
+    assert success is True
     assert out.rstrip() == "message"
 
     #
@@ -338,7 +338,7 @@ def test_forkserver_preimport(tmpdir):
     finally:
         spawner.close()
 
-    assert success == False
+    assert success is False
     assert out.startswith('asv: benchmark runner crashed')
 
     #
@@ -359,7 +359,7 @@ def test_forkserver_preimport(tmpdir):
     finally:
         spawner.close()
 
-    assert success == True
+    assert success is True
     assert out.startswith('Traceback')
 
 
