@@ -1,39 +1,12 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import os
+import textwrap
 from os.path import abspath, dirname, join
 
-import six
-import textwrap
-
-from asv import config
-
 from . import tools
-from .tools import example_results
-
-import pytest
-
 
 BENCHMARK_DIR = abspath(join(dirname(__file__), 'example_results'))
 MACHINE_FILE = abspath(join(dirname(__file__), 'asv-machine.json'))
 
-@pytest.fixture
-def show_fixture(tmpdir, example_results):
-    tmpdir = six.text_type(tmpdir)
-    os.chdir(tmpdir)
-
-    conf = config.Config.from_json(
-        {'results_dir': example_results,
-         'repo': tools.generate_test_repo(tmpdir).path,
-         'project': 'asv',
-         'environment_type': "shouldn't matter what"})
-
-    return conf
-
 
 def test_show(capsys, show_fixture):
     conf = show_fixture
