@@ -1,53 +1,29 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import warnings
-from itertools import product
-import math
 import random
+import warnings
+import math
+from itertools import product
 
 import pytest
+from asv_runner.statistics import (compute_stats, LaplacePosterior, quantile, quantile_ci,
+                                   binom_pmf, get_err)
 
-from asv.util import inf
-import asv.statistics as statistics
+from asv import statistics
 
 
-try:
-    import numpy as np
-    HAS_NUMPY = True
-except ImportError:
-    HAS_NUMPY = False
-
-
-try:
-    from scipy import integrate, special, stats
-    HAS_SCIPY = True
-except ImportError:
-    HAS_SCIPY = False
-
-
-try:
-    from rpy2 import robjects
-    HAS_RPY2 = True
-except ImportError:
-    HAS_RPY2 = False
-
-
-@pytest.mark.skipif(not HAS_NUMPY, reason="Requires numpy")
 def test_compute_stats():
+    np = pytest.importorskip("numpy")
     np.random.seed(1)
 
-    assert statistics.compute_stats([], 1) == (None, None)
-    assert statistics.compute_stats([15.0], 1) == (
-        15.0, {'ci_99_a': -inf, 'ci_99_b': inf,
+    assert compute_stats([], 1) == (None, None)
+    assert compute_stats([15.0], 1) == (
+        15.0, {'ci_99_a': -math.inf, 'ci_99_b': math.inf,
                'number': 1, 'q_25': 15.0, 'q_75': 15.0, 'repeat': 1})
 
     for nsamples, true_mean in product([10, 50, 250], [0, 0.3, 0.6]):
         samples = np.random.randn(nsamples) + true_mean
-        result, stats = statistics.compute_stats(samples, 42)
+        result, stats = compute_stats(samples, 42)
 
         assert stats['repeat'] == len(samples)
         assert stats['number'] == 42
@@ -60,21 +36,21 @@ def test_compute_stats():
         w = 12.0 * np.std(samples) / np.sqrt(len(samples))
         assert ci[1] - ci[0] < w
 
-        err = statistics.get_err(result, stats)
+        err = get_err(result, stats)
         iqr = np.percentile(samples, 75) - np.percentile(samples, 25)
-        assert np.allclose(err, iqr/2)
+        assert np.allclose(err, iqr / 2)
 
 
-@pytest.mark.skipif(not HAS_NUMPY, reason="Requires numpy")
 def test_is_different():
+    np = pytest.importorskip("numpy")
     np.random.seed(1)
 
     # Smoke test is_different
     for true_mean, n, significant in [(0.01, 10, False), (0.05, 100, True), (0.1, 10, True)]:
         samples_a = 0 + 0.1 * np.random.rand(n)
         samples_b = true_mean + 0.1 * np.random.rand(n)
-        result_a, stats_a = statistics.compute_stats(samples_a, 1)
-        result_b, stats_b = statistics.compute_stats(samples_b, 1)
+        _, stats_a = compute_stats(samples_a, 1)
+        _, stats_b = compute_stats(samples_b, 1)
         assert statistics.is_different(None, None, stats_a, stats_b) == significant
         assert statistics.is_different(samples_a, samples_b, stats_a, stats_b) == significant
 
@@ -107,7 +83,7 @@ def _check_ci(estimator, sampler, nsamples=300):
 
     for size, alpha in product(sizes, alphas):
         samples = []
-        for k in range(nsamples):
+        for _ in range(nsamples):
             z = sampler(size)
             m, ci = estimator(z, alpha)
             a, b = ci
@@ -119,10 +95,9 @@ def _check_ci(estimator, sampler, nsamples=300):
         yield size, alpha, alpha_got
 
 
-@pytest.mark.skipif(not HAS_NUMPY, reason="Requires numpy")
 def test_quantile_ci():
     # Test the confidence intervals
-
+    np = pytest.importorskip("numpy")
     scale = 2.5
 
     def sample_exp(size):
@@ -136,28 +111,28 @@ def test_quantile_ci():
     np.random.seed(1)
 
     for sampler in [sample_exp, sample_normal]:
-        cis = _check_ci(lambda z, alpha: statistics.quantile_ci(z, 0.5, alpha),
+        cis = _check_ci(lambda z, alpha: quantile_ci(z, 0.5, alpha),
                         sampler, nsamples=300)
-        atol = 5/300
+        atol = 5 / 300
         for size, alpha, alpha_got in cis:
             if size < 20:
-                assert 0 <= alpha_got <= 1.2*alpha
+                assert 0 <= alpha_got <= 1.2 * alpha
             else:
-                assert 0.5*alpha - atol <= alpha_got <= 1.1*alpha + atol
+                assert 0.5 * alpha - atol <= alpha_got <= 1.1 * alpha + atol
 
 
 def test_quantile_ci_small():
     # Small samples should give infinite ci
     for n in range(1, 7):
         sample = list(range(n))
-        m, ci = statistics.quantile_ci(sample, 0.5, 0.99)
-        assert ci[0] == -inf
-        assert ci[1] == inf
+        _, ci = quantile_ci(sample, 0.5, 0.99)
+        assert ci[0] == -math.inf
+        assert ci[1] == math.inf
 
 
 def test_quantile_ci_r():
     # Compare to R
-    x = [-2.47946614, -1.49595963, -1.02812482, -0.76592323, -0.09452743,  0.10732743,
+    x = [-2.47946614, -1.49595963, -1.02812482, -0.76592323, -0.09452743, 0.10732743,
          0.27798342, 0.50173779, 0.57829823, 0.60474948, 0.94695675, 1.20159789]
 
     # quantile(x, type=7, prob=p)
@@ -168,11 +143,11 @@ def test_quantile_ci_r():
     # asht::quantileTest(x, prob=p, conf.level=0.8)$conf.int
     ci_20_80_e = [-2.47946614, -0.09452743]
     ci_50_80_e = [-0.7659232, 0.5782982]
-    ci_80_80_e = [0.5017378, 1.2015979,]
+    ci_80_80_e = [0.5017378, 1.2015979]
 
-    q_20, ci_20_80 = statistics.quantile_ci(x, 0.2, 0.8)
-    q_50, ci_50_80 = statistics.quantile_ci(x, 0.5, 0.8)
-    q_80, ci_80_80 = statistics.quantile_ci(x, 0.8, 0.8)
+    q_20, ci_20_80 = quantile_ci(x, 0.2, 0.8)
+    q_50, ci_50_80 = quantile_ci(x, 0.5, 0.8)
+    q_80, ci_80_80 = quantile_ci(x, 0.8, 0.8)
 
     assert q_20 == pytest.approx(q_20_e)
     assert q_50 == pytest.approx(q_50_e)
@@ -183,43 +158,32 @@ def test_quantile_ci_r():
     assert ci_80_80 == pytest.approx(ci_80_80_e, abs=1e-4)
 
 
-@pytest.mark.skipif(not HAS_NUMPY, reason="Requires numpy")
 def test_quantile():
+    np = pytest.importorskip("numpy")
+
     np.random.seed(1)
     x = np.random.randn(50)
     for q in np.linspace(0, 1, 300):
         expected = np.percentile(x, 100 * q)
-        got = statistics.quantile(x.tolist(), q)
+        got = quantile(x.tolist(), q)
         assert np.allclose(got, expected), q
 
 
-@pytest.mark.skipif(not HAS_SCIPY, reason="Requires scipy")
-def test_lgamma():
-    x = np.arange(1, 5000)
-
-    expected = special.gammaln(x)
-    got = np.vectorize(statistics.lgamma)(x)
-
-    assert np.allclose(got, expected, rtol=1e-12, atol=0)
-    assert np.isnan(statistics.lgamma(1.2))
-
-
-@pytest.mark.skipif(not HAS_SCIPY, reason="Requires scipy")
 def test_binom_pmf():
+    np = pytest.importorskip("numpy")
+    stats = pytest.importorskip("scipy.stats")
+
     p = np.linspace(0, 1, 7)
-    k = np.arange(0, 40, 5)[:,None]
-    n = np.arange(0, 40, 5)[:,None,None]
-
+    k = np.arange(0, 40, 5)[:, None]
+    n = np.arange(0, 40, 5)[:, None, None]
     expected = stats.binom.pmf(k, n, p)
-    got = np.vectorize(statistics.binom_pmf)(n, k, p)
-
+    got = np.vectorize(binom_pmf)(n, k, p)
     assert np.allclose(got, expected, rtol=1e-12, atol=0)
 
 
-@pytest.mark.skipif(not HAS_NUMPY, reason="Requires numpy")
 def test_laplace_posterior_ci():
     # Test the LaplacePosterior confidence intervals
-
+    np = pytest.importorskip("numpy")
     scale = 2.5
 
     def get_z_exp(size):
@@ -236,16 +200,16 @@ def test_laplace_posterior_ci():
     np.random.seed(41)
 
     def estimator(z, alpha):
-        c = statistics.LaplacePosterior(z.tolist())
-        a, b = c.ppf(alpha/2), c.ppf(1 - alpha/2)
+        c = LaplacePosterior(z.tolist())
+        a, b = c.ppf(alpha / 2), c.ppf(1 - alpha / 2)
         a = min(c.mle, a)  # force MLE inside CI
         b = max(c.mle, b)
         return c.mle, (a, b)
 
     for sampler in [get_z_exp, get_z_normal]:
         cis = _check_ci(estimator, sampler, nsamples=300)
-        atol = 5/300
-        for size, alpha, alpha_got in cis:
+        atol = 5 / 300
+        for _, alpha, alpha_got in cis:
             if sampler == get_z_exp:
                 # Result should be ok for the assumed distribution
                 rtol = 0.25
@@ -261,27 +225,27 @@ def test_laplace_posterior_basic():
 
     # even
     y = [1, 2, 3, 4, 5, 6][::-1]
-    c = statistics.LaplacePosterior(y)
+    c = LaplacePosterior(y)
     assert abs(c.mle - 3.5) < 1e-8
 
     # odd
     y = [1, 2, 3, 4, 5][::-1]
-    c = statistics.LaplacePosterior(y)
+    c = LaplacePosterior(y)
     assert abs(c.mle - 3) < 1e-8
 
     # check pdf vs cdf
     sx = 200
-    dx = 1.0/sx
+    dx = 1.0 / sx
     cdf = 0
-    for jx in range(-10*sx, 10*sx):
-        cdf += c.pdf(dx*jx) * dx
-        got = c.cdf(dx*jx)
+    for jx in range(-10 * sx, 10 * sx):
+        cdf += c.pdf(dx * jx) * dx
+        got = c.cdf(dx * jx)
         assert abs(cdf - got) < 3e-3
     assert abs(cdf - 1.0) < 1e-3
 
     # large input (must not cause overflows)
     y = list(range(500))
-    c = statistics.LaplacePosterior(y)
+    c = LaplacePosterior(y)
     assert abs(c.mle - 249.5) < 1e-8
     assert abs(c.cdf(249.5) - 0.5) < 1e-8
 
@@ -300,7 +264,7 @@ def test_laplace_posterior_basic():
 
     # check zero variance
     y = [1, 1, 1, 1, 1]
-    c = statistics.LaplacePosterior(y)
+    c = LaplacePosterior(y)
     assert c.mle == 1
     assert c.pdf(1 - 1e-5) == 0
     assert c.pdf(1 + 1e-5) == 0
@@ -310,24 +274,27 @@ def test_laplace_posterior_basic():
 
     # one item
     y = [1]
-    c = statistics.LaplacePosterior(y)
+    c = LaplacePosterior(y)
     assert c.cdf(1 - 1e-5) == 0
     assert c.cdf(1 + 1e-5) == 1.0
 
     # zero items
     with pytest.raises(ValueError):
-        statistics.LaplacePosterior([])
+        LaplacePosterior([])
+
 
-    
-@pytest.mark.skipif(not HAS_SCIPY, reason="Requires scipy")
 def test_laplace_posterior_cdf():
     # Test the LaplacePosterior cdf vs pdf
+    np = pytest.importorskip("numpy")
+    integrate = pytest.importorskip("scipy.integrate")
+
     np.random.seed(1)
     y = np.random.randn(15).tolist()
 
-    c = statistics.LaplacePosterior(y)
+    c = LaplacePosterior(y)
 
-    num_cdf = lambda t: integrate.quad(c.pdf, -np.inf, t, limit=1000)[0]
+    def num_cdf(t):
+        return integrate.quad(c.pdf, -np.inf, t, limit=1000)[0]
 
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", category=integrate.IntegrationWarning)
@@ -388,9 +355,10 @@ def test_mann_whitney_u_cdf():
     check_table(5, tbl)
 
 
-@pytest.mark.skipif(not HAS_SCIPY, reason="Requires scipy")
 def test_mann_whitney_u_scipy():
     # Scipy only has the large-sample limit...
+    np = pytest.importorskip("numpy")
+    stats = pytest.importorskip("scipy.stats")
 
     def check(x, y):
         u0, p0 = stats.mannwhitneyu(x, y, alternative='two-sided', use_continuity=False)
@@ -434,11 +402,12 @@ def test_mann_whitney_u_basic():
     b = [2.5]
     u, p = statistics.mann_whitney_u(a, b, method='exact')
     assert u == 0
-    assert p == pytest.approx(2/3, abs=0, rel=1e-10)
+    assert p == pytest.approx(2 / 3, abs=0, rel=1e-10)
 
 
-@pytest.mark.skipif(not HAS_RPY2, reason="Requires rpy2")
 def test_mann_whitney_u_R():
+    robjects = pytest.importorskip("rpy2.robjects")
+
     random.seed(1)
 
     a = list(range(30))
